/****************************************/
/*  CopyDialog.c body            	*/
/*					*/
/*  Description : Manage Copy (copy of  */
/*                files) widget		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

/********************************************************************/
/* TODO :                                                           */
/* - choice to adapt or not link + link on directory problem        */
/* - Check if files already exist before writting and use a dialog  */
/*   to let the choice to the user                                  */
/* - modify mode need when overwritting a file                      */
/* - threads for this dialog                                        */
/* - check all C function error code                                */
/* - make the button Cancel work good (stop and remove de current   */
/*   file)                                                          */
/********************************************************************/

static void copy_dialog_class_init(CopyDialogClass *pCopyDialogClass);
static void copy_dialog_init(CopyDialog *pCopyDialog);
static void destroy_cb(GtkWidget *Widget, CopyDialog *pCopyDialog);
static void cancel_cb(GtkWidget *Widget, CopyDialog *pCopyDialog);

/*************************************************************/
guint copy_dialog_get_type ()
{
  static guint copy_dialog_type = 0;

  if (!copy_dialog_type)
  {
    GtkTypeInfo copy_dialog_info =
    {
       "CopyDialog",
       sizeof (CopyDialog),
       sizeof (CopyDialogClass),
       (GtkClassInitFunc)  copy_dialog_class_init,
       (GtkObjectInitFunc) copy_dialog_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     copy_dialog_type = gtk_type_unique(gnome_dialog_get_type(), &copy_dialog_info);
  }
  return copy_dialog_type;
}
/*************************************************************/

/*************************************************************/
static void copy_dialog_class_init(CopyDialogClass *class)
{
  /* here we create the new signals if we want */
}
/*************************************************************/

void copy_current_file(CopyDialog *pCopyDialog);

static void list_all_we_need_to_copy_sub(CopyDialog *pCopyDialog,FileData *pFileData)
{ struct dirent **namelist;
  gint   nb_entry;
  gint   entry_pos = 0;
  gchar  *string;
  FileData *vFileData;
  
  string = g_malloc(strlen(pFileData->path)+strlen(pFileData->name)+2);
  strcpy(string,pFileData->path);
  strcat(string,pFileData->name);

  /* open the content of a dir */
  nb_entry = scandir(string, &namelist, 0, 0);
  if (nb_entry < 0)
  {g_print("COPY_DIALOG, list_all_we_need_to_copy_sub : dir not found\n"); return;} /* THE DIRECTORY DOESN'T EXIST */

  strcat(string,"/");
  while(entry_pos < nb_entry)
  {
    /* do something only if it is not . or .. */
    if(strcmp(namelist[entry_pos]->d_name,".") &&
       strcmp(namelist[entry_pos]->d_name,".."))
    {   
      vFileData = file_data_new_from_file(string,namelist[entry_pos]->d_name,NULL,NULL);
      file_data_list_append(pCopyDialog->AllCopyList, vFileData);
      pCopyDialog->current_all_file_total_size_cpt += vFileData->stat.st_size;
      if(S_ISDIR(vFileData->stat.st_mode))
        list_all_we_need_to_copy_sub(pCopyDialog,vFileData);
      file_data_free(vFileData);
    }  
    entry_pos++;
  }
  
  g_free(string);
  g_free(namelist);
}

static void list_all_we_need_to_copy(CopyDialog *pCopyDialog)
{/* gchar *string;*/

  pCopyDialog->AllCopyList = file_data_list_new();

  file_data_list_init(pCopyDialog->CopyList);
  while(!file_data_list_is_end(pCopyDialog->CopyList))
  { 
    /* do something only if it is not . or .. */
    if(strcmp(file_data_list_current(pCopyDialog->CopyList)->name,".") &&
       strcmp(file_data_list_current(pCopyDialog->CopyList)->name,".."))
    {
      file_data_list_append(pCopyDialog->AllCopyList, file_data_list_current(pCopyDialog->CopyList));
      pCopyDialog->current_all_file_total_size_cpt +=
	file_data_list_current(pCopyDialog->CopyList)->stat.st_size;
      if(S_ISDIR(file_data_list_current(pCopyDialog->CopyList)->stat.st_mode))
        list_all_we_need_to_copy_sub(pCopyDialog,file_data_list_current(pCopyDialog->CopyList));
    }
    file_data_list_next(pCopyDialog->CopyList);
  }
}

static gchar *suppress_start(gchar *start,gchar *string)
{ guint16 pos = 0;
  guint16 end_pos;
  /*g_print("Start : %s, String : %s\n",start,string);*/

  while ((start[pos]) && (string[pos]) && (start[pos] == string[pos])){pos++;}
  /* if we are not at the end of start, that mean that string doesn't
     start like the "start" string. */  
  if(start[pos] != 0) {return(NULL);}
  return(&string[pos]);
}

/***********************************/
/* Take care about copying devices */
void copy_device(CopyDialog *pCopyDialog,FileData *SrcFile,gchar *DstFile)
{ struct stat vstat;
  gchar *vSrcFile;
  
  vSrcFile = g_strconcat(SrcFile->path,SrcFile->name,NULL);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),0.0);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_of),vSrcFile);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_to),DstFile);
  g_free(vSrcFile);
  while(gtk_events_pending()) gtk_main_iteration();
  if(pCopyDialog->exit_wanted == TRUE){ return; }
  lstat(vSrcFile, &vstat);
  mknod(DstFile, vstat.st_mode, vstat.st_dev);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),1.0);
  while(gtk_events_pending()) gtk_main_iteration();
  if(pCopyDialog->exit_wanted == TRUE){ remove(DstFile); return; }
}
/***********************************/

/*******************************************************/
/* Take care about copying directory (not the content) */
void copy_dir(CopyDialog *pCopyDialog,FileData *SrcFile,gchar *DstFile)
{ struct stat vstat;
  gchar *vSrcFile;
  
  vSrcFile = g_strconcat(SrcFile->path,SrcFile->name,NULL);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),0.0);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_of),vSrcFile);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_to),DstFile);
  g_free(vSrcFile);
  while(gtk_events_pending()) gtk_main_iteration();  
  if(pCopyDialog->exit_wanted == TRUE){ return; }
  mkdir(DstFile, SrcFile->stat.st_mode | S_IWUSR);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),1.0);
  while(gtk_events_pending()) gtk_main_iteration();
  if(pCopyDialog->exit_wanted == TRUE){ rmdir(DstFile); return; }
}
/*******************************************************/

/*****************************************/
/* Take care about copying symbolic link */
void copy_symlink(CopyDialog *pCopyDialog,FileData *SrcFile,gchar *DstFile)
{ struct stat vstat;
  gchar *vSrcFile;
  
  vSrcFile = g_strconcat(SrcFile->path,SrcFile->name,NULL);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),0.0);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_of),vSrcFile);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_to),DstFile);
  g_free(vSrcFile);
  while(gtk_events_pending()) gtk_main_iteration();
  if(pCopyDialog->exit_wanted == TRUE){ return; }
  symlink(SrcFile->link_to_file, DstFile);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),1.0);
  while(gtk_events_pending()) gtk_main_iteration();
  if(pCopyDialog->exit_wanted == TRUE){ remove(DstFile); return; }
}
/*****************************************/

/*****************************************/
/* Take care about copying regular files */
void copy_regfile(CopyDialog *pCopyDialog,FileData *SrcFile,gchar *DstFile)
{
  gchar *buf;
  guint bytes_read;
  guint bytes_written;
  int   file_handle_src;
  int   file_handle_dst;
  struct stat vstat;
  gfloat percent;
  gint   wait_cpt;
  gchar *vSrcFile;
  
  vSrcFile = g_strconcat(SrcFile->path,SrcFile->name,NULL);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),0.0);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_of),vSrcFile);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_to),DstFile);
  while(gtk_events_pending()) gtk_main_iteration();
  if(pCopyDialog->exit_wanted == TRUE){ g_free(vSrcFile); return; }
  
  lstat(vSrcFile, &vstat);
  buf = g_malloc(vstat.st_blksize);
  pCopyDialog->current_file_size_cpt = 0;
  pCopyDialog->current_file_total_size_cpt = vstat.st_size;
  file_handle_src = open(vSrcFile, O_RDONLY);
  file_handle_dst = open(DstFile, O_CREAT | O_WRONLY | O_TRUNC, vstat.st_mode | S_IWUSR);
  do
  {	
    bytes_read = read(file_handle_src,buf,vstat.st_blksize);
    bytes_written = write(file_handle_dst,buf,bytes_read);
    /* error to do when bytes_read != bytes_written */
    pCopyDialog->current_file_size_cpt += bytes_read;
    /* I don't like division by 0 */
    if(pCopyDialog->current_file_total_size_cpt == 0)
      percent = 1.0;
    else
      percent = ((gfloat)pCopyDialog->current_file_size_cpt) / 
        ((gfloat)pCopyDialog->current_file_total_size_cpt);
    /*g_print("percent %f\n",percent);*/
    if(percent > 1.0) percent = 1.0; /* file has grown */
    gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),percent);
    while(gtk_events_pending()) gtk_main_iteration();
    /* if the user what to close the dialog */
    if(pCopyDialog->exit_wanted == TRUE)
    { close(file_handle_dst); close(file_handle_src);
      remove(DstFile);
      g_free(buf); g_free(vSrcFile); return;
    }
    
  } while(bytes_read == vstat.st_blksize);
  close(file_handle_dst);
  close(file_handle_src);

  g_free(buf);
  g_free(vSrcFile);
}
/*****************************************/

static void dialog_yes_cb(GtkWidget *Widget, CopyDialog *pCopyDialog)
{ 
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pCopyDialog->apply_all_check)) == TRUE)
    pCopyDialog->overwrite_all = TRUE; 
}

static void dialog_no_cb(GtkWidget *Widget, CopyDialog *pCopyDialog)
{
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pCopyDialog->apply_all_check)) == TRUE)
    pCopyDialog->overwrite_none = TRUE;
}

void copy_current_file(CopyDialog *pCopyDialog)
{ FileData *vFileData;
  FileData *current;
  gchar *DstPath;
  gchar *NewFile;
  gfloat percent;
  GtkWidget *widget;
  GtkWidget *table;
  gchar *string,*string2,*string3;

  vFileData = file_data_list_current(pCopyDialog->AllCopyList);
  DstPath = g_strconcat(pCopyDialog->DestPath,
   suppress_start(pCopyDialog->SrcPath,vFileData->path),NULL);
  NewFile = g_strconcat(DstPath,vFileData->name,NULL);

  if((pCopyDialog->overwrite_all == FALSE) &&
     g_file_exists(NewFile))
  {
    /*g_print("file : %s already exist\n",NewFile);*/
    if(pCopyDialog->overwrite_none == TRUE)
    { /* the user already says to never overwrite */
      g_free(NewFile); g_free(DstPath); return; 
    } else if(pCopyDialog->overwrite_all == FALSE) {
      /* we are going to ask the user to make a choice */
      pCopyDialog->question = gnome_dialog_new("Replace ?",
        GNOME_STOCK_BUTTON_YES,GNOME_STOCK_BUTTON_NO,
	GNOME_STOCK_BUTTON_CANCEL,NULL);
      /* button Yes */
      gnome_dialog_button_connect(GNOME_DIALOG(pCopyDialog->question),0,
        GTK_SIGNAL_FUNC(dialog_yes_cb),pCopyDialog);
      /* button No */
      gnome_dialog_button_connect(GNOME_DIALOG(pCopyDialog->question),1,
        GTK_SIGNAL_FUNC(dialog_no_cb),pCopyDialog);

      table = gtk_table_new(5,5,FALSE);
      gtk_widget_show(table);
      gtk_box_pack_start_defaults(GTK_BOX(GNOME_DIALOG(pCopyDialog->question)->vbox),
        table);
	
      string = g_strdup_printf("Target file \"%s\" already exist\n"\
                                 "Replace current by new one ?\n",NewFile);
      widget = gtk_label_new(string);
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 0, 5, 0, 1);

      g_free(string);
	
      /* the ligne "New     : last modification : %s" */
      widget = left_align_label("New");
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 0, 1, 1, 2);

      widget = gtk_label_new(":");
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 1, 2, 1, 2);

      widget = left_align_label("last modification");
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 2, 3, 1, 2);

      widget = gtk_label_new(":");
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 3, 4, 1, 2);

      string2 = g_strdup(ctime(&vFileData->stat.st_mtime));
      string2[strlen(string2)-1] = 0;

      widget = left_align_label(string2);
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 4, 5, 1, 2);

      g_free(string2);
	
      /* the ligne "          file size         : %d bytes" */	
      widget = left_align_label("file size");
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 2, 3, 2, 3);

      widget = gtk_label_new(":");
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 3, 4, 2, 3);

      string2 = g_strdup_printf("%d bytes",vFileData->stat.st_size);

      widget = left_align_label(string2);
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 4, 5, 2, 3);

      g_free(string2);
	
      /* the ligne "Current : last modification : %s" */
      widget = left_align_label("Current");
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 0, 1, 3, 4);

      widget = gtk_label_new(":");
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 1, 2, 3, 4);

      widget = left_align_label("last modification");
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 2, 3, 3, 4);

      widget = gtk_label_new(":");
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 3, 4, 3, 4);

      /* the ligne "          file size         : %d" */
      widget = left_align_label("file size");
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 2, 3, 4, 5);

      widget = gtk_label_new(":");
      gtk_widget_show(widget);
      gtk_table_attach_defaults(GTK_TABLE(table),widget, 3, 4, 4, 5);

      if((current = file_data_new_from_file(DstPath,vFileData->name,NULL,NULL)) != NULL)
      {
        string2 = g_strdup(ctime(&current->stat.st_mtime));
        string2[strlen(string2)-1] = 0;

        widget = left_align_label(string2);
        gtk_widget_show(widget);
        gtk_table_attach_defaults(GTK_TABLE(table),widget, 4, 5, 3, 4);

        g_free(string2);

        string2 = g_strdup_printf("%d bytes",current->stat.st_size);

        widget = left_align_label(string2);
        gtk_widget_show(widget);
        gtk_table_attach_defaults(GTK_TABLE(table),widget, 4, 5, 4, 5);

        g_free(string2);

        file_data_free(current);

      } else { /* if we can't stat the current file */
	widget = left_align_label("?");
	gtk_widget_show(widget);
	gtk_table_attach_defaults(GTK_TABLE(table),widget, 4, 5, 3, 4);

        widget = left_align_label("?");
        gtk_widget_show(widget);
        gtk_table_attach_defaults(GTK_TABLE(table),widget, 4, 5, 4, 5);
      }
			
      pCopyDialog->apply_all_check = gtk_check_button_new_with_label("apply for all");
      gtk_widget_show(pCopyDialog->apply_all_check);
      gtk_box_pack_start_defaults(GTK_BOX(GNOME_DIALOG(pCopyDialog->question)->vbox),
        pCopyDialog->apply_all_check);
      gnome_dialog_set_close(GNOME_DIALOG(pCopyDialog->question),TRUE);
      switch(gnome_dialog_run(GNOME_DIALOG(pCopyDialog->question)))
      {
        case -1 : 
	  /* like if the user choose abort */
	  /*g_print("Fenetre detruite\n");*/
	  gnome_dialog_close(GNOME_DIALOG(pCopyDialog));
	  break;
	case 0  :
	  /*g_print("Yes\n");*/
	  break;
	case 1  :
	  /*g_print("No\n");*/
	  g_free(NewFile); g_free(DstPath); return;
	  break;
	case 2  :
	  /*g_print("Abort\n");*/
	  gnome_dialog_close(GNOME_DIALOG(pCopyDialog));
	  break;
	default :
	  g_print("Impossible choice for the question dialog\n");
      }
    }
  }

  if(S_ISDIR(vFileData->stat.st_mode))
    copy_dir(pCopyDialog,vFileData,NewFile);
  else if(S_ISLNK(vFileData->stat.st_mode))
    copy_symlink(pCopyDialog,vFileData,NewFile);
  else if(S_ISBLK(vFileData->stat.st_mode) || S_ISCHR(vFileData->stat.st_mode))
    copy_device(pCopyDialog,vFileData,NewFile);
  else if (!S_ISFIFO(vFileData->stat.st_mode) &&
           !S_ISSOCK(vFileData->stat.st_mode))
    copy_regfile(pCopyDialog,vFileData,NewFile);
  if(pCopyDialog->exit_wanted == TRUE){ g_free(NewFile); g_free(DstPath); return; }
    
  pCopyDialog->file_added_func(NewFile);
   
  g_free(NewFile);

  /* update the total progress bar */
  pCopyDialog->current_all_file_size_cpt += vFileData->stat.st_size;
  /* I don't like division by 0 */
  if(pCopyDialog->current_all_file_total_size_cpt == 0)
    percent = 1.0;
  else
    percent = ((gfloat)pCopyDialog->current_all_file_size_cpt) / 
        ((gfloat)pCopyDialog->current_all_file_total_size_cpt);

  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->all_file_progress_bar),percent);
  while(gtk_events_pending()) gtk_main_iteration();
   
  /*g_print("copy of : %s%s, to %s%s.\n",vFileData->path,vFileData->name,
    DstPath,vFileData->name);*/
  g_free(DstPath);
}


/******************************************/
/* called when everything has been copied */
static gint copy_dialog_exit_timer_cb(CopyDialog *pCopyDialog)
{ gtk_timeout_remove (pCopyDialog->exit_timer);
  gnome_dialog_close (GNOME_DIALOG(pCopyDialog));
}
/******************************************/

/* given by copy_dialog_new as a parameter for copy_dialog_init */
static FileDataList *current_CopyList;
static gchar        *current_DestPath;
static gchar        *current_SrcPath;
static void (* current_file_added_func) (gchar *NewFile);

/*************************************************************/
static void copy_dialog_init(CopyDialog *pCopyDialog)
{ GtkWidget *label;
  GtkWidget *table;
    
  /* take parameters in account */
  pCopyDialog->CopyList = current_CopyList;
  pCopyDialog->DestPath = current_DestPath;
  pCopyDialog->SrcPath  = current_SrcPath;
  pCopyDialog->file_added_func  = current_file_added_func;
  
  pCopyDialog->exit_wanted = FALSE;
  pCopyDialog->overwrite_all  = FALSE; /* wait the user choice */
  pCopyDialog->overwrite_none = FALSE; /* wait the user choice */
  
  gtk_window_set_title(GTK_WINDOW(pCopyDialog),"Copy");
  /* add the CANCEL button */
  gnome_dialog_append_buttons(GNOME_DIALOG(pCopyDialog),
    GNOME_STOCK_BUTTON_CANCEL,NULL);

  gtk_signal_connect(GTK_OBJECT(pCopyDialog), "destroy",
    GTK_SIGNAL_FUNC(destroy_cb),pCopyDialog);    
    
  /* boutton Cancel */
  gnome_dialog_button_connect(GNOME_DIALOG(pCopyDialog),0,
    GTK_SIGNAL_FUNC(cancel_cb),pCopyDialog);
    
  gtk_widget_realize (GTK_WIDGET(pCopyDialog));

  gtk_box_set_homogeneous(GTK_BOX(GNOME_DIALOG(pCopyDialog)->vbox),FALSE);

  /* table for packing all things */
  table = gtk_table_new(4,2,FALSE);
  gtk_widget_show(table);  
  
  gtk_table_set_row_spacings(GTK_TABLE(table),10);
  gtk_table_set_col_spacings(GTK_TABLE(table),10);  
  
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pCopyDialog)->vbox),
    table,TRUE, TRUE, 2);

  /* copy of */
  label = gtk_label_new("Copy of :");
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(label);  
  gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 0, 1);
  pCopyDialog->copy_of = gtk_label_new("");
  gtk_label_set_justify (GTK_LABEL(pCopyDialog->copy_of),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(pCopyDialog->copy_of);  
  gtk_table_attach_defaults(GTK_TABLE(table),pCopyDialog->copy_of,
    1, 2, 0, 1);

  /* copy to */
  label = gtk_label_new("to :");
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 1, 2);
  pCopyDialog->copy_to = gtk_label_new("");
  gtk_label_set_justify (GTK_LABEL(pCopyDialog->copy_to),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(pCopyDialog->copy_to);
  gtk_table_attach_defaults(GTK_TABLE(table),pCopyDialog->copy_to,
    1, 2, 1, 2);

  /* current file progress bar */
  label = gtk_label_new("current");
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 2, 3);
  pCopyDialog->file_progress_bar = gtk_progress_bar_new();
    /* show the percentage */
  gtk_progress_set_show_text(GTK_PROGRESS(pCopyDialog->file_progress_bar),
    TRUE);  
  gtk_widget_show(pCopyDialog->file_progress_bar);
  gtk_table_attach_defaults(GTK_TABLE(table),pCopyDialog->file_progress_bar,
    1, 2, 2, 3);

  /* total file progress bar */
  label = gtk_label_new("total");
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);    
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 3, 4);
  pCopyDialog->all_file_progress_bar = gtk_progress_bar_new();
    /* show the percentage */
  gtk_progress_set_show_text(GTK_PROGRESS(pCopyDialog->all_file_progress_bar),
    TRUE);
  gtk_widget_show(pCopyDialog->all_file_progress_bar);
  gtk_table_attach_defaults(GTK_TABLE(table),pCopyDialog->all_file_progress_bar,
    1, 2, 3, 4);
  
  gtk_widget_realize(GTK_WIDGET(pCopyDialog));
  /* we must do this here if we want to see something */
  gtk_widget_show(GTK_WIDGET(pCopyDialog));
  
/*  while(gtk_events_pending())
    gtk_main_iteration();

  if(pCopyDialog->exit_wanted == TRUE) return;*/

  /* find all we have really to copy (directory...) */
  list_all_we_need_to_copy(pCopyDialog);

  while(gtk_events_pending())
    gtk_main_iteration();
    
  if(pCopyDialog->exit_wanted == TRUE) return;

  file_data_list_init(pCopyDialog->AllCopyList);
  while(!file_data_list_is_end(pCopyDialog->AllCopyList))
  { 
    copy_current_file(pCopyDialog);
    if(pCopyDialog->exit_wanted == TRUE) return;
    file_data_list_next(pCopyDialog->AllCopyList);
  }

  /* when copy is finished, close the dialog */
  pCopyDialog->exit_timer = gtk_timeout_add(0,
    (GtkFunction)(&copy_dialog_exit_timer_cb), pCopyDialog);
}
/*************************************************************/

/***************************************************/
/* called when the user click on the cancel button */
static void cancel_cb(GtkWidget *Widget, CopyDialog *pCopyDialog)
{ gnome_dialog_close (GNOME_DIALOG(pCopyDialog)); }
/***************************************************/

/***************************************/
/* close when the dialog box is closed */
static void destroy_cb(GtkWidget *Widget, CopyDialog *pCopyDialog)
{
  /* if the dialog was closed, let the other know that we have stopped */
  pCopyDialog->exit_wanted = TRUE;
  
  file_data_list_free(pCopyDialog->CopyList);
  file_data_list_free(pCopyDialog->AllCopyList);  
  g_free(pCopyDialog->DestPath);
  g_free(pCopyDialog->SrcPath);
}
/***************************************/

/**************************************************/
/* Create a new GFileCopy widget                  */
/* GFileCopy keep a copy of CopyList and DestDir. */
GtkWidget *copy_dialog_new (FileDataList *CopyList,gchar *DestPath,
  void (* file_added_func) (gchar *NewFile))
{
  file_data_list_init(CopyList);
  /* if nothing to copy, exit*/
  if(file_data_list_is_end(CopyList)) return(NULL);
  /* if srcpath = dstpath, exit*/
  if(!strcmp(file_data_list_current(CopyList)->path,DestPath)) return(NULL);
  current_SrcPath = g_strdup(file_data_list_current(CopyList)->path);

  current_CopyList = file_data_list_copy(CopyList);
  current_DestPath = g_strdup(DestPath); 
  current_file_added_func = file_added_func;
  return( GTK_WIDGET(gtk_type_new( copy_dialog_get_type() )) );
}
/**************************************************/
