/****************************************/
/*  CopyDialog.c headers         	*/
/*					*/
/*  Description : Manage Copy (copy of  */
/*                files) widget		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#ifndef __COPY_DIALOG_H__
#define __COPY_DIALOG_H__

/* to define a new widget */
#define COPY_DIALOG(obj)          GTK_CHECK_CAST (obj, copy_dialog_get_type(),CopyDialog)
#define COPY_DIALOG_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, copy_dialog_get_type(), CopyDialogClass)
#define IS_COPY_DIALOG(obj)       GTK_CHECK_TYPE (obj, copy_dialog_get_type ())

typedef struct _CopyDialog      CopyDialog;
typedef struct _CopyDialogClass CopyDialogClass;

/* the structure of a CopyDialog */
struct _CopyDialog
{
  GnomeDialog	Main;
  /* list of all file to copy */
  FileDataList  *CopyList;
  /* destination directory */
  gchar         *DestPath;
  /* Source base directory */
  gchar         *SrcPath;
  
  GtkWidget     *copy_of;
  GtkWidget     *copy_to;
  GtkWidget     *current_size;
  GtkWidget     *current_speed;
  GtkWidget     *total_size;
  GtkWidget     *total_file;
  GtkWidget     *file_progress_bar;
  GtkWidget     *all_file_progress_bar;

  /* dialog used for asking question if file already exist */
  GtkWidget *question;
  /* apply for all check button of the question dialog */
  GtkWidget *apply_all_check;

  /* if TRUE, we overwrite all existing file by new one without asking */
  gboolean      overwrite_all;  /* TO DO */
  /* if TRUE, we never overwrite all existing file by new one without asking */
  gboolean      overwrite_none; /* TO DO */

  /* nb of bytes we have copied */
  guint64       current_file_size_cpt;
  /* nb of bytes to copy */
  guint64       current_file_total_size_cpt;

  /* nb of bytes we have copied */
  guint64       current_all_file_size_cpt;
  /* nb of bytes to copy */
  guint64       current_all_file_total_size_cpt;
  
  /* if TRUE, we must stop copying */
  gboolean exit_wanted;
  
  gint          timer;
  gint          exit_timer; 
  
  void (* file_added_func) (gchar *NewFile);
  
  /* list of all thing we need to copy */
  FileDataList  *AllCopyList;  
};

struct _CopyDialogClass
{
   GnomeDialogClass parent_class;

   /* Add here all the signal we want */
};

guint copy_dialog_get_type ();
/* Create a new CopyDialog widget                  */
/* CopyDialog keep a copy of CopyList and DestPath. */
GtkWidget *copy_dialog_new (FileDataList *CopyList,gchar *DestPath,
  void (* file_added_func) (gchar *NewFile));

#endif /* __COPY_DIALOG_H__ */
