/****************************************/
/*  FileProperties.c body               */
/*					*/
/*  Description : Dialog with the       */
/*                properties of a file  */
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

/*******************************************************************/
/* TO DO                                                           */
/* - support for more than one file properties with a smart rights */
/*   change system                                                 */
/* - drag & drop : dropping a file on this dialog must give        */
/*   its properties                                                */
/* - beeing allowed to apply change "recursively" when there is    */
/*   directory                                                     */
/* - check all c functions error code                              */
/*******************************************************************/

static void file_properties_class_init(FilePropertiesClass *pFilePropertiesClass);
static void file_properties_init(FileProperties *pFileProperties);

/*************************************************************/
guint file_properties_get_type ()
{
  static guint file_properties_type = 0;

  if (!file_properties_type)
  {
    GtkTypeInfo file_properties_info =
    {
       "FileProperties",
       sizeof (FileProperties),
       sizeof (FilePropertiesClass),
       (GtkClassInitFunc)  file_properties_class_init,
       (GtkObjectInitFunc) file_properties_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     file_properties_type = gtk_type_unique(gnome_dialog_get_type(), &file_properties_info);
  }
  return file_properties_type;
}
/*************************************************************/

/*************************************************************/
enum { FILE_MODE_CHANGED_SIGNAL, LAST_SIGNAL};
static gint file_properties_signals[LAST_SIGNAL] = { 0 };

static void two_file_data_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*two_file_data_marshal_func)(FileProperties *, FileData *, FileData *,gpointer);
    two_file_data_marshal_func rfunc = (two_file_data_marshal_func) func;
    (*rfunc) (FILE_PROPERTIES(object),
	      GTK_VALUE_POINTER(args[0]),GTK_VALUE_POINTER(args[1]),func_data );
}

static void file_properties_class_init(FilePropertiesClass *class)
{
  /* here we create the new signals if we want */
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) class;

  file_properties_signals[FILE_MODE_CHANGED_SIGNAL] = gtk_signal_new
            ("file_mode_changed",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (FilePropertiesClass,file_mode_changed),
            two_file_data_marshal, GTK_TYPE_NONE, 2,GTK_TYPE_POINTER,
	    GTK_TYPE_POINTER);
	    
   gtk_object_class_add_signals(object_class, 
     file_properties_signals, LAST_SIGNAL);
   class->file_mode_changed = NULL;
}
/*************************************************************/

/* called when the file_properties dialog is destroy */
static void destroy_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  file_data_list_free(pFileProperties->file_list);
  gtk_grab_remove(GTK_WIDGET(pFileProperties));
}

static void ur_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_ur = TRUE;
  pFileProperties->ur = !pFileProperties->ur;
}

static void uw_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_uw = TRUE;
  pFileProperties->uw = !pFileProperties->uw;
}

static void ux_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_ux = TRUE;
  pFileProperties->ux = !pFileProperties->ux;
}

static void gr_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_gr = TRUE;
  pFileProperties->gr = !pFileProperties->gr;
}

static void gw_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_gw = TRUE;
  pFileProperties->gw = !pFileProperties->gw;
}

static void gx_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_gx = TRUE;
  pFileProperties->gx = !pFileProperties->gx;
}

static void or_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_or = TRUE;
  pFileProperties->or = !pFileProperties->or;
}

static void ow_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_ow = TRUE;
  pFileProperties->ow = !pFileProperties->ow;
}

static void ox_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_ox = TRUE;
  pFileProperties->ox = !pFileProperties->ox;
}

static void setuid_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_setuid = TRUE;
  pFileProperties->setuid = !pFileProperties->setuid;
}

static void setgid_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_setgid = TRUE;
  pFileProperties->setgid = !pFileProperties->setgid;
}

static void setsticky_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_setsticky = TRUE;
  pFileProperties->setsticky = !pFileProperties->setsticky;
}


/* called when Cancel is pressed */
static void cancel_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  gnome_dialog_close(GNOME_DIALOG(pFileProperties));
}

/* called when Ok is pressed */
static void ok_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{ FileData *vFileData,*vFileData2;
  gchar *string;
  gchar *string2;
  mode_t mode;
  gint  uid,gid;
  gchar *error_msg;
  GtkWidget *error_dialog;
  grp *vgrp;
  own *vown;
  gint compt;
  gboolean is_num;

  file_data_list_init(pFileProperties->file_list);
  vFileData = file_data_list_current(pFileProperties->file_list);

  string2 = g_strconcat(vFileData->path,
    gtk_entry_get_text(GTK_ENTRY(pFileProperties->name)),NULL);

  /* if the file was a link check if the link have changed */
  if(S_ISLNK(vFileData->stat.st_mode) && 
     (strcmp(vFileData->link_to_file,gtk_entry_get_text(GTK_ENTRY(pFileProperties->link_to)))))
  {   
    string = g_strconcat(vFileData->path,vFileData->name,NULL);

    /* remove the last link before creating a new one */
    if(remove(string) == -1){g_print("remove ERROR. Case NOT DONE.\n");}
    if(symlink(gtk_entry_get_text(GTK_ENTRY(pFileProperties->link_to)),string2) == -1)
    {g_print("symlink ERROR. Case NOT DONE.\n");}

    g_free(string);
  /* if the name have changed and we are not a link */  
  } else if(strcmp(vFileData->name,gtk_entry_get_text(GTK_ENTRY(pFileProperties->name)))) {
  
    string = g_strconcat(vFileData->path,vFileData->name,NULL);

    if(rename(string, string2) == -1)
    {  /*g_print("rename ERROR. Case NOT DONE.\n");*/
      switch(errno)
      {
        case EISDIR :
          error_msg = "newpath is an existing directory, but oldpath is\n"\
                      "not a directory.";
          break;
        case EXDEV :
          error_msg = "oldpath and newpath are not on the same filesystem.";
          break;
        case ENOTEMPTY :
          error_msg = "newpath is a non-empty directory.";
          break;
        case EBUSY :
          error_msg = "newpath exists and is the current working\n"\
	              "directory or root directory of some process.";
          break;
        case EEXIST :
          error_msg = "The new pathname contained a path prefix\n"\
	              "of the old.";
          break;
        case EINVAL :
          error_msg = "An attempt was made to make a directory a\n"\
	              "subdirectory of itself.";
          break;
        case EMLINK :
          error_msg = "oldpath already has the maximum number of links to\n"\
                      "it, or it was a directory and the directory\n"\
	      	      "containing newpath has the maximum number of links.";
          break;
        case ENOTDIR :
          error_msg = "A component used as a directory in oldpath or\n"\
	              "newpath is not, in fact, a directory.";
          break;
        default :
          error_msg = "Unknown ERROR";
          break;
      }
      error_dialog = gnome_message_box_new(error_msg,GNOME_MESSAGE_BOX_ERROR,
        GNOME_STOCK_BUTTON_OK,NULL);
      gnome_dialog_set_close(GNOME_DIALOG(error_dialog),TRUE);
      gnome_dialog_run(GNOME_DIALOG(error_dialog));
      g_free(string); g_free(string2);
      return;
    }
    g_free(string);
  }

  if(pFileProperties->change_ur||pFileProperties->change_uw||
     pFileProperties->change_ux||pFileProperties->change_gr||
     pFileProperties->change_gw||pFileProperties->change_gx||
     pFileProperties->change_or||pFileProperties->change_ow||
     pFileProperties->change_ox||pFileProperties->change_setuid||
     pFileProperties->change_setgid||pFileProperties->change_setsticky)
  {   
    /*g_print("Rights have changed\n");*/
    /* create the new rights */
    mode = 0;
    if(pFileProperties->ur) mode |= S_IRUSR;
    if(pFileProperties->uw) mode |= S_IWUSR;
    if(pFileProperties->ux) mode |= S_IXUSR;
    if(pFileProperties->gr) mode |= S_IRGRP;
    if(pFileProperties->gw) mode |= S_IWGRP;
    if(pFileProperties->gx) mode |= S_IXGRP;
    if(pFileProperties->or) mode |= S_IROTH;    
    if(pFileProperties->ow) mode |= S_IWOTH;    
    if(pFileProperties->ox) mode |= S_IXOTH;
    if(pFileProperties->setuid) mode |= S_ISUID;
    if(pFileProperties->setgid) mode |= S_ISGID;
    if(pFileProperties->setsticky) mode |= S_ISVTX;

    mode |= vFileData->stat.st_mode & (~0x1FF);

    if(chmod(string2, mode) == -1)
    {  /*g_print("chmod ERROR when changing rights. Case NOT DONE.\n");*/
      switch(errno)
      {
        case EPERM :
	  error_msg = "The effective UID does not match the owner of\n"
	              "the file, and is not zero.";
          break;
        case EROFS :
	  error_msg = "The named file resides on a read-only file system.";
          break;
        case EFAULT :
	  error_msg = "path points outside your accessible address space.";
          break;
        case ENAMETOOLONG :
          error_msg = "path is too long.";
          break;
        case ENOENT :
	  error_msg = "The file does not exist.";
          break;
        case ENOMEM :
	  error_msg = "Insufficient kernel memory was available.";
          break;
        case ENOTDIR :
	  error_msg = "A component of the path prefix is not a directory.";
          break;
        case EACCES :
	  error_msg = "Search permission is denied on a component\n"\
	              "of the path prefix.";
          break;
        case ELOOP :
	  error_msg = "Too many symbolic links were encountered in\n"\
                      "resolving path.";
          break;
        case EIO :
	  error_msg = "An I/O error occurred.";
          break;
       default :
         error_msg = "Unknown ERROR";
         break;
      }
      error_dialog = gnome_message_box_new(error_msg,GNOME_MESSAGE_BOX_ERROR,
        GNOME_STOCK_BUTTON_OK,NULL);
      gnome_dialog_set_close(GNOME_DIALOG(error_dialog),TRUE);
      gnome_dialog_run(GNOME_DIALOG(error_dialog));
      g_free(string2);
      return;
    }
  }/* else  
    g_print("Rights have NOT changed\n");*/

  string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(pFileProperties->owner)->entry));
  compt = 0;
  is_num = TRUE;
  while((string[compt] != 0) && (is_num == TRUE))
  { is_num = (string[compt] >= '0') && (string[compt] <= '9'); compt++; }
  if((is_num == TRUE) && (strlen(string) > 0)) {
    uid = atoi(string);
  } else {
    vown = owner_find_by_name(pFileProperties->owner_list,string);
    if(vown == NULL)
    {
      error_dialog = gnome_message_box_new("Unknown Owner",GNOME_MESSAGE_BOX_ERROR,
        GNOME_STOCK_BUTTON_OK,NULL);
      gnome_dialog_set_close(GNOME_DIALOG(error_dialog),TRUE);
      gnome_dialog_run(GNOME_DIALOG(error_dialog));
      g_free(string2);
      return;
    }
    uid = vown->nb;
  }
  
  string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(pFileProperties->group)->entry));
  compt = 0;
  is_num = TRUE;
  while((string[compt] != 0) && (is_num == TRUE))
  { is_num = (string[compt] >= '0') && (string[compt] <= '9'); compt++; }
  if((is_num == TRUE) && (strlen(string) > 0)) {
    gid = atoi(string);
  } else {
    vgrp = group_find_by_name(pFileProperties->group_list,string);
    if(vgrp == NULL)
    {
      error_dialog = gnome_message_box_new("Unknown Group",GNOME_MESSAGE_BOX_ERROR,
        GNOME_STOCK_BUTTON_OK,NULL);
      gnome_dialog_set_close(GNOME_DIALOG(error_dialog),TRUE);
      gnome_dialog_run(GNOME_DIALOG(error_dialog));
      g_free(string2);
      return;
    }
    gid = vgrp->nb;
  }

  if((uid != vFileData->stat.st_uid) || (gid != vFileData->stat.st_gid))
  {
    if(chown(string2, uid, gid) == -1)
    {
      switch(errno)
      {
        case EPERM :
	  error_msg = "The effective UID does not match the owner of the\n"\
                      "file, and is not zero; or the owner or group were\n"\
                      "specified incorrectly.";
          break;
        case EROFS :
	  error_msg = "The named file resides on a read-only file system.";

        case EFAULT :
	  error_msg = "path points outside your accessible address space.";
          break;
        case ENAMETOOLONG :
          error_msg = "path is too long.";
          break;
        case ENOENT :
	  error_msg = "The file does not exist.";
          break;
        case ENOMEM :
	  error_msg = "Insufficient kernel memory was available.";
          break;
        case ENOTDIR :
	  error_msg = "A component of the path prefix is not a directory.";
          break;
        case EACCES :
	  error_msg = "Search permission is denied on a component of"\
	              "the path prefix.";
          break;
        case ELOOP :
	  error_msg = "Too many symbolic links were encountered in"\
                      "resolving path.";
          break;
       default :
         error_msg = "Unknown ERROR";
         break;
      }
      error_dialog = gnome_message_box_new(error_msg,GNOME_MESSAGE_BOX_ERROR,
        GNOME_STOCK_BUTTON_OK,NULL);
      gnome_dialog_set_close(GNOME_DIALOG(error_dialog),TRUE);
      gnome_dialog_run(GNOME_DIALOG(error_dialog));
      g_free(string2);
      return;
    }
  }

  vFileData2 = file_data_new_from_file(vFileData->path,gtk_entry_get_text(GTK_ENTRY(pFileProperties->name)),
    pFileProperties->file_type_list,pFileProperties->owner_list);

  g_print("New. Name : %s%s.\n",vFileData2->path,vFileData2->name);
  gtk_signal_emit(GTK_OBJECT(pFileProperties),
    file_properties_signals[FILE_MODE_CHANGED_SIGNAL],vFileData,vFileData2);

  file_data_free(vFileData2);

  g_free(string2);
  gnome_dialog_close(GNOME_DIALOG(pFileProperties));
}

static FileDataList *current_FileDataList;
static FileTypeList *current_FileTypeList;
static Owner        *current_Owner;
static Group        *current_Group;

/*************************************************************/
static void file_properties_init(FileProperties *pFileProperties)
{ FileData *vFileData;
  GtkWidget *label;
  GtkWidget *table;
  gchar *string, *string2;
  GtkWidget *check_but;
  GList     *strings = NULL; /* use for fill the combo boxes */
  own *vown;
  grp *vgrp;
  
  /* take parameters in account */
  pFileProperties->file_list       = current_FileDataList;
  pFileProperties->file_type_list  = current_FileTypeList;
  pFileProperties->owner_list      = current_Owner;
  pFileProperties->group_list      = current_Group;

  file_data_list_init(pFileProperties->file_list);
  /* if there is no file in the list, exit */
  if(file_data_list_is_end(pFileProperties->file_list) == TRUE) return;

  gtk_window_set_title(GTK_WINDOW(pFileProperties),"Properties");

  /* add the OK and CANCEL buttons */
  gnome_dialog_append_buttons(GNOME_DIALOG(pFileProperties),
    GNOME_STOCK_BUTTON_OK,GNOME_STOCK_BUTTON_CANCEL,NULL);

  gtk_signal_connect (GTK_OBJECT (pFileProperties), 
		      "destroy",
		      (GtkSignalFunc) destroy_cb,
		      pFileProperties);

  /* connect ok_cb to the Ok button of the dialog */
  gnome_dialog_button_connect(GNOME_DIALOG(pFileProperties),0,
    GTK_SIGNAL_FUNC(ok_cb),pFileProperties);

  /* connect cancel_cb to the Cancel button of the dialog */
  gnome_dialog_button_connect(GNOME_DIALOG(pFileProperties),1,
    GTK_SIGNAL_FUNC(cancel_cb),pFileProperties);

  vFileData = file_data_list_current(pFileProperties->file_list);

  /* if it is a link, need "link to" field */
  if(S_ISLNK(vFileData->stat.st_mode))
    table = gtk_table_new(4,2,FALSE);
  else
    table = gtk_table_new(3,2,FALSE);
  gtk_table_set_row_spacings(GTK_TABLE(table),4);
  gtk_widget_show(table);

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pFileProperties)->vbox),
    table,TRUE,TRUE,4);

  label = gtk_label_new("type :");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,0,1);  

  if(S_ISLNK(vFileData->stat.st_mode))
    label = left_align_label("symbolic link"); else
  if(S_ISDIR(vFileData->stat.st_mode))
    label = left_align_label("directory"); else
  if(S_ISSOCK(vFileData->stat.st_mode))
    label = left_align_label("socket"); else  
  if(S_ISBLK(vFileData->stat.st_mode))
    label = left_align_label("block device"); else  
  if(S_ISCHR(vFileData->stat.st_mode))
    label = left_align_label("character device"); else  
  if(S_ISFIFO(vFileData->stat.st_mode))
    label = left_align_label("FIFO"); else  
    label = left_align_label("regular file");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,0,1);
      
  label = gtk_label_new("path :");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,1,2);  
  
  label = left_align_label(vFileData->path);
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,1,2);

  label = gtk_label_new("name :");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,2,3);
  
  pFileProperties->name = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(pFileProperties->name),vFileData->name);
  gtk_widget_show(pFileProperties->name);
  gtk_table_attach_defaults(GTK_TABLE(table),pFileProperties->name,1,2,2,3);

  /* unactive name if we are not the owner or root */
  if((owner_current_user(pFileProperties->owner_list)->nb != 0) && 
     (vFileData->stat.st_uid != owner_current_user(pFileProperties->owner_list)->nb))
  { gtk_widget_set_sensitive(pFileProperties->name,FALSE); }

  if(S_ISLNK(vFileData->stat.st_mode))
  {
    if(vFileData->extra_info & FILE_DATA_IS_BROKEN_SYMB)
      label = gtk_label_new("broken link to :");
    else    
      label = gtk_label_new("link to :");
    gtk_widget_show(label);
    gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,3,4);

    pFileProperties->link_to = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(pFileProperties->link_to),vFileData->link_to_file);
    gtk_widget_show(pFileProperties->link_to);
    gtk_table_attach_defaults(GTK_TABLE(table),pFileProperties->link_to,1,2,3,4);

    /* unactive link to if we are not the owner or root */
    if((owner_current_user(pFileProperties->owner_list)->nb != 0) && 
       (vFileData->stat.st_uid != owner_current_user(pFileProperties->owner_list)->nb))
    { gtk_widget_set_sensitive(pFileProperties->link_to,FALSE); }
  }
  
  /* the file size */
  string2 = g_malloc(22+8);
  sprintf(string2,"size : %u",vFileData->stat.st_size);
  label = left_align_label(string2);
  gtk_widget_show(label);

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pFileProperties)->vbox),
    label,TRUE,TRUE,4);
  
  /* all dates informations */
  table = gtk_table_new(3,2,FALSE);
  gtk_table_set_row_spacings(GTK_TABLE(table),4);
  gtk_table_set_col_spacings(GTK_TABLE(table),4);
  gtk_widget_show(table);

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pFileProperties)->vbox),
    table,TRUE,TRUE,4);

  label = left_align_label("last access");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,0,1);
  
  string = ctime(&vFileData->stat.st_atime);
  string[strlen(string)-1] = 0;
  string2 = g_strconcat(": ",string,NULL);
  label = left_align_label(string2);
  g_free(string2);
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,0,1);

  label = left_align_label("last modif");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,1,2);
  
  string = ctime(&vFileData->stat.st_mtime);
  string[strlen(string)-1] = 0;
  string2 = g_strconcat(": ",string,NULL);
  label = left_align_label(string2);
  g_free(string2);
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,1,2);

  label = left_align_label("last status change");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,2,3);
  
  string = ctime(&vFileData->stat.st_ctime);
  string[strlen(string)-1] = 0;  
  string2 = g_strconcat(": ",string,NULL);
  label = left_align_label(string2);
  g_free(string2);
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,2,3);

  /* if it is a link, don't display rights (useless) */
  if(!S_ISLNK(vFileData->stat.st_mode))
  {
  /* rights */
  table = gtk_table_new(4,4,FALSE);
  gtk_table_set_row_spacings(GTK_TABLE(table),4);
  gtk_table_set_col_spacings(GTK_TABLE(table),4);
  gtk_widget_show(table);

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pFileProperties)->vbox),
    table,TRUE,TRUE,4);

  /* owner rights */
  label = gtk_label_new("Owner");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,0,1);

  check_but = gtk_check_button_new_with_label("read");
  if(vFileData->stat.st_mode & S_IRUSR)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->ur = TRUE;}
  else pFileProperties->ur = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,0,1,1,2);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(ur_cb),pFileProperties);
  pFileProperties->check_ur = check_but;
  	   
  check_but = gtk_check_button_new_with_label("write");
  if(vFileData->stat.st_mode & S_IWUSR)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->uw = TRUE;}
  else pFileProperties->uw = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,0,1,2,3);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(uw_cb),pFileProperties);
  pFileProperties->check_uw = check_but;
  
  check_but = gtk_check_button_new_with_label("exec");
  if(vFileData->stat.st_mode & S_IXUSR)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->ux = TRUE;}
  else pFileProperties->ux = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,0,1,3,4);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(ux_cb),pFileProperties);
  pFileProperties->check_ux = check_but;

  /* group rights */
  label = gtk_label_new("Group");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,0,1);

  check_but = gtk_check_button_new_with_label("read");
  if(vFileData->stat.st_mode & S_IRGRP)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->gr = TRUE;}
  else pFileProperties->gr = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,1,2,1,2);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(gr_cb),pFileProperties);
  pFileProperties->check_gr = check_but;

  check_but = gtk_check_button_new_with_label("write");
  if(vFileData->stat.st_mode & S_IWGRP)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->gw = TRUE;}
  else pFileProperties->gw = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,1,2,2,3);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(gw_cb),pFileProperties);
  pFileProperties->check_gw = check_but;
  
  check_but = gtk_check_button_new_with_label("exec");
  if(vFileData->stat.st_mode & S_IXGRP)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->gx = TRUE;}
  else pFileProperties->gx = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,1,2,3,4);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(gx_cb),pFileProperties);
  pFileProperties->check_gx = check_but;

  /* others rights */
  label = gtk_label_new("Others");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,2,3,0,1);

  check_but = gtk_check_button_new_with_label("read");
  if(vFileData->stat.st_mode & S_IROTH)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->or = TRUE;}
  else pFileProperties->or = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,2,3,1,2);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(or_cb),pFileProperties);
  pFileProperties->check_or = check_but;

  check_but = gtk_check_button_new_with_label("write");
  if(vFileData->stat.st_mode & S_IWOTH)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->ow = TRUE;}
  else pFileProperties->ow = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,2,3,2,3);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(ow_cb),pFileProperties);
  pFileProperties->check_ow = check_but;

  check_but = gtk_check_button_new_with_label("exec");
  if(vFileData->stat.st_mode & S_IXOTH)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->ox = TRUE;}
  else pFileProperties->ox = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,2,3,3,4);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(ox_cb),pFileProperties);
  pFileProperties->check_ox = check_but;

  /* special rights */
  label = gtk_label_new("Special");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,3,4,0,1);

  check_but = gtk_check_button_new_with_label("set UID");
  if(vFileData->stat.st_mode & S_ISUID)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->setuid = TRUE;}
  else pFileProperties->setuid = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,3,4,1,2);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(setuid_cb),pFileProperties);
  pFileProperties->check_setuid = check_but;

  check_but = gtk_check_button_new_with_label("set GID");
  if(vFileData->stat.st_mode & S_ISGID)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->setgid = TRUE;}
  else pFileProperties->setgid = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,3,4,2,3);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(setgid_cb),pFileProperties);
  pFileProperties->check_setgid = check_but;

  check_but = gtk_check_button_new_with_label("set Sticky");
  if(vFileData->stat.st_mode & S_ISVTX)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->setsticky = TRUE;}
  else pFileProperties->setsticky = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,3,4,3,4);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(setsticky_cb),pFileProperties);
  pFileProperties->check_setsticky = check_but;

  pFileProperties->change_ur = FALSE;
  pFileProperties->change_uw = FALSE;
  pFileProperties->change_ux = FALSE;
  pFileProperties->change_gr = FALSE;
  pFileProperties->change_gw = FALSE;
  pFileProperties->change_gx = FALSE;
  pFileProperties->change_or = FALSE;
  pFileProperties->change_ow = FALSE;
  pFileProperties->change_ox = FALSE;
  pFileProperties->change_setuid = FALSE;
  pFileProperties->change_setgid = FALSE;
  pFileProperties->change_setsticky = FALSE;

  /* unactive all rights if we are not the owner or root */
  if((owner_current_user(pFileProperties->owner_list)->nb != 0) && 
     (vFileData->stat.st_uid != owner_current_user(pFileProperties->owner_list)->nb))
  {
    gtk_widget_set_sensitive(pFileProperties->check_ur,FALSE);
    gtk_widget_set_sensitive(pFileProperties->check_uw,FALSE);
    gtk_widget_set_sensitive(pFileProperties->check_ux,FALSE);
    
    gtk_widget_set_sensitive(pFileProperties->check_gr,FALSE);
    gtk_widget_set_sensitive(pFileProperties->check_gw,FALSE);
    gtk_widget_set_sensitive(pFileProperties->check_gx,FALSE);
  
    gtk_widget_set_sensitive(pFileProperties->check_or,FALSE);
    gtk_widget_set_sensitive(pFileProperties->check_ow,FALSE);
    gtk_widget_set_sensitive(pFileProperties->check_ox,FALSE);

    gtk_widget_set_sensitive(pFileProperties->check_setuid,FALSE);
    gtk_widget_set_sensitive(pFileProperties->check_setgid,FALSE);
    gtk_widget_set_sensitive(pFileProperties->check_setsticky,FALSE);
  }
  
  } /* end of test if it is a SYMB LINK or not */

  table = gtk_table_new(2,2,FALSE);
  gtk_table_set_row_spacings(GTK_TABLE(table),4);
  gtk_table_set_col_spacings(GTK_TABLE(table),4);
  gtk_widget_show(table);

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pFileProperties)->vbox),
    table,TRUE,TRUE,4);

  /* display the owner */
  pFileProperties->owner = gtk_combo_new();
  gtk_widget_show(pFileProperties->owner);
  /* unactive the owner choice if we are not root */
  if(owner_current_user(pFileProperties->owner_list)->nb != 0)
  { gtk_widget_set_sensitive(pFileProperties->owner,FALSE); }
  label = gtk_label_new("Owner"); gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,0,1);
  gtk_table_attach_defaults(GTK_TABLE(table),pFileProperties->owner,1,2,0,1);

  owner_init(pFileProperties->owner_list);
  while(owner_is_end(pFileProperties->owner_list) == FALSE)
  {
    strings = g_list_append(strings,owner_current(pFileProperties->owner_list)->name);
    owner_next(pFileProperties->owner_list);
  }
  if(strings != NULL)
  {
    gtk_combo_set_popdown_strings(GTK_COMBO(pFileProperties->owner),strings);
    g_list_free(strings);
  }
  vown = owner_find_by_nb(pFileProperties->owner_list,vFileData->stat.st_uid);
  if(vown != NULL) {
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(pFileProperties->owner)->entry),vown->name);
  } else {
    string = g_strdup_printf("%d",vFileData->stat.st_uid);
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(pFileProperties->owner)->entry),string);
    g_free(string);
  }
  
  /* display the group */
  pFileProperties->group = gtk_combo_new();
  gtk_widget_show(pFileProperties->group);
  /* unactive the group choice if we are not root */
  if((owner_current_user(pFileProperties->owner_list)->nb != 0) && 
     (vFileData->stat.st_uid != owner_current_user(pFileProperties->owner_list)->nb))
  { gtk_widget_set_sensitive(pFileProperties->group,FALSE); }
  label = gtk_label_new("Group"); gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,1,2);
  gtk_table_attach_defaults(GTK_TABLE(table),pFileProperties->group,1,2,1,2);

  strings = NULL;
  group_init(pFileProperties->group_list);
  while(group_is_end(pFileProperties->group_list) == FALSE)
  {
    strings = g_list_append(strings,group_current(pFileProperties->group_list)->name);
    group_next(pFileProperties->group_list);
  }
  if(strings != NULL)
  {
    gtk_combo_set_popdown_strings(GTK_COMBO(pFileProperties->group),strings);
    g_list_free(strings);
  }
  
  vgrp = group_find_by_nb(pFileProperties->group_list,vFileData->stat.st_gid);
  if(vgrp != NULL) {
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(pFileProperties->group)->entry),vgrp->name);
  } else {
    string = g_strdup_printf("%d",vFileData->stat.st_gid);
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(pFileProperties->group)->entry),string);
    g_free(string);
  }
  
  /* we want that user can do nothing else when this dialog appear */
  gtk_grab_add(GTK_WIDGET(pFileProperties));
}
/*************************************************************/

/***************************************/
/* Create a new file properties dialog */
GtkWidget *file_properties_new(FileDataList *pFileDataList,
  FileTypeList *pFileTypeList,Owner *pOwner,Group *pGroup)
{
  current_FileDataList = file_data_list_copy(pFileDataList);
  current_FileTypeList = pFileTypeList;
  current_Owner        = pOwner;
  current_Group        = pGroup;

  return( GTK_WIDGET(gtk_type_new( file_properties_get_type() )) );
}
/***************************************/
