/****************************************/
/*  FileProperties.c headers            */
/*					*/
/*  Description : Dialog with the       */
/*                properties of a file  */
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#ifndef __FILE_PROPERTIES_H__
#define __FILE_PROPERTIES_H__

/* to define a new widget */
#define FILE_PROPERTIES(obj)          GTK_CHECK_CAST (obj, file_properties_get_type(),FileProperties)
#define FILE_PROPERTIES_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, file_properties_get_type(), FilePropertiesClass)
#define IS_FILE_PROPERTIES(obj)       GTK_CHECK_TYPE (obj, file_properties_get_type ())

typedef struct _FileProperties       FileProperties;
typedef struct _FilePropertiesClass  FilePropertiesClass;

struct _FileProperties
{
  GnomeDialog main;
  
  /* list of FileData of all file we want the properties */
  /* this is a copy of the one given by the maker */
  FileDataList *file_list;
  
  /* display rights */
  gboolean ur,uw,ux, gr,gw,gx, or,ow,ox, setuid,setgid,setsticky;
  
  /* to says that the user have change the rights have changed */
  /* usefull when we have the properties of more than 1 file   */
  gboolean change_ur,change_uw,change_ux, change_gr,change_gw,change_gx,
           change_or,change_ow,change_ox, 
	   change_setuid,change_setgid,change_setsticky;

  /* All check button for each right */
  GtkWidget *check_ur,*check_uw,*check_ux, *check_gr,*check_gw,*check_gx,
            *check_or,*check_ow,*check_ox, 
	    *check_setuid,*check_setgid,*check_setsticky;
  
  /* gtk_entry where the user see/change the filename */
  GtkWidget *name;
  /* gtk_entry where the user see/change where a symblink go */
  /* exist only if the file is a link */
  GtkWidget *link_to;
  
  GtkWidget *owner;
  GtkWidget *group;
  
  /* filetype list. DON'T change it this is NOT A COPY of the one
     given by the maker */
  FileTypeList *file_type_list;
  /* owner list. DON'T change it this is NOT A COPY of the one
     given by the maker */
  Owner    *owner_list;
  /* group list. DON'T change it this is NOT A COPY of the one
     given by the maker */
  Group    *group_list;
};

struct _FilePropertiesClass
{
  GnomeDialogClass parent_class;

  /* Add here all the signal we want */

  /* signal called when the mode of a file as been changed */
  void (* file_mode_changed) (FileProperties *pFileProperties,FileData *SrcFile,
    FileData *DstFile);
};

guint file_properties_get_type ();

/* Create a new file properties dialog */
/* pFileDataList is copied so you can modify yours but NOT 
   pFileType and pOwner. If pOwner and pFileType are NULL,
   DstFile (in signal file_mode_changed) will not have attached
   the corresponding FileType and right FILE_DATA_IS_ALLOW_TO_EXEC
   will not be tested. */
GtkWidget *file_properties_new(FileDataList *pFileDataList,
  FileTypeList *pFileTypeList,Owner *pOwner,Group *pGroup);

#endif /* __FILE_PROPERTIES_H__ */
