/****************************************/
/*  FileType.c headers                  */
/*					*/
/*  Description : FileType take care of */
/*		  file type reconization*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#ifndef __FILE_TYPE_H__
#define __FILE_TYPE_H__

typedef struct _FileType FileType;

/* possible return error */
#define FILE_TYPE_PARAM_ERROR             1
#define FILE_TYPE_UNKOW_MUST_BE_FILE_TYPE 2
#define FILE_TYPE_LOADING_ICON_ERROR      3

/* MUST BE defined types */
enum{NORMAL_DIRECTORY,		/* MUST ALWAYS BE THE FIRST */
     NORMAL_OPEN_DIRECTORY,
     CURRENT_DIRECTORY,
     CURRENT_OPEN_DIRECTORY,
     PAST_DIRECTORY,
     PAST_OPEN_DIRECTORY,
     DEFAULT_TYPE,
/* MUST BE PRESENT TYPE ARE BETWEEN NORMAL_DIRECTORY AND LAST_MUST_BE_TYPE */
     LAST_MUST_BE_TYPE,
/* facultative type */
     BROKEN_LINK,
     EXECUTABLE,
     SOCKET,
     FIFO,
     BLOCK_DEVICE,
     CHARACTER_DEVICE,
     USER_DEFINED};		/* MUST ALWAYS BE THE LAST */

/* string corresponding to mode enum */
extern gchar *mode_str[];

struct _FileType
{
  gboolean  is_link;		/* say if the type is define for a link*/
                                /* or a regular file. TRUE = for a link*/

  gchar     *type_str;          /* only for USER_DEFINED mode,else NULL*/
                                /* ex : *.gif                          */

  gchar     *program_str;       /* ex : xv. Can be NULL if no program  */
                                /* is associated.                      */

  gchar     *icon_name;         /* icon filename to use with this type */
  				/* if icon_name == NULL, internal icon */
				/* is used. If it don't start with '/' */
				/* this is the relative name with base */
				/* dir : GFILEDIR/icons/               */

  guint8    mode;               /* define the FileType mode : SOCKET...*/

  GdkPixmap *pixmap;            /* pixmap made with icon               */

  GdkBitmap *mask;              /* mask made with icon                 */
};


#define file_type_is_link(pFileType)   pFileType->is_link
#define file_type_mode(pFileType)      pFileType->mode
#define file_type_icon_name(pFileType) pFileType->icon_name

/* set a new icon_path for icon not full path define            */
/* exemple of path : /usr/share/pixmap/ (always ended by a '/') */
void file_type_set_icon_path(gchar *new_path);

/* return the icon path */
extern gchar *icon_path;
#define file_type_get_icon_path() icon_path

/* Create a new file type.                                     */
/* Return : NULL if impossible and gfile_errno can be set to : */
/* FILE_TYPE_PARAM_ERROR                                       */
/* FILE_TYPE_UNKOW_MUST_BE_FILE_TYPE                           */
/* FILE_TYPE_LOADING_ICON_ERROR                                */
FileType *file_type_new(guint8 mode, gchar *type_str, gchar *program_str,
  gchar *icon_name, gboolean is_link, GdkPixmap *pixmap, GdkBitmap *mask);

/* Free a file type. */
void file_type_free(FileType *pFileType);

/* set the parent application to avoid warning when using */
/* gdk_pixmap_create_from_xpm_d                           */
void file_type_set_parent(GtkWidget *parent);

/* Return a copy of pFileType                            */
/* or NULL if impossible and gfile_errno can be set to : */
/* FILE_TYPE_PARAM_ERROR                                 */
/* FILE_TYPE_UNKOW_MUST_BE_FILE_TYPE                     */
/* FILE_TYPE_LOADING_ICON_ERROR                          */
FileType *file_type_copy(FileType *pFileType);

/* Return the mode corresponding to mode string (NORMAL_DIRECTORY, */
/* NORMAL_OPEN_DIRECTORY..) or -1 if not found                     */
guint8 file_type_str_2_mode(gchar *mode_string);

#endif /* __FILE_TYPE_H__ */
