/*******************************************/
/*					   */
/*  The main of GFile.			   */
/*					   */
/*  GFile is a File manager for Gnome      */
/*					   */
/*  Written by : Daniel Lacroix	           */
/*  GFile is under the GPL2 licence        */
/*******************************************/

/*******************************************************************/
/* TO DO                                                           */
/* - major clean up :)                                             */
/* - check all case when switching from one mode to another to see */
/*   if there is no problem about the active part.                 */
/* - only one Preference dialog allowed at a time                  */
/*******************************************************************/

#include "GFile.h"

/* file where we save the file type list. (file in GFILEDIR) */
#define FILE_TYPE_FILE "gfile_file_type"
/* directory where icons are on. (directory in GFILEDIR) */
#define ICON_PATH      "icons"
/* gfile look specific file */
#define RC_FILE	       "gfilerc"
/* file where we save the send to list. (file in GFILEDIR) */
#define SEND_TO_FILE "gfile_send_to_commands"

/* GFILEDIR indicate where is the GFile base directory */
/* it is based on the environment variable GFILEDIR    */
/* This is ALWAYS ended by a '/'                       */
gchar *GFILEDIR;

GtkWidget *vbox;
/* need for MODE_LIST_LIST and MODE_TREE_LIST */
GtkWidget *hpaned;
/* for the right file list */
GtkWidget *RightPart;
/* For the left part (tree, list or nothing) */
GtkWidget *LeftPart;

/* List of all file in copy/cut mode */
FileDataList *copy_file_list = NULL;
gboolean cut_mode; /* if TRUE,we are in a cut else a copy */

/* all preferences = file "gfile_preference" */
Preference gfile_preference;

/* to know if the user is now working in the left or the right part */
/* like that, if copy, del... is called job will be done in the good*/
/* part (LEFT_PART or RIGHT_PART)                                   */
guint8 active_part = LEFT_PART;

/* list of all known file type */
FileTypeList *in_use_file_type_list;

/* list of all known group on this computer */
struct _Group *in_use_group_list;

/* list of all known users on this computer */
struct _Owner *in_use_owner_list;

/* list of all known send to commands */
SendToList *in_use_send_to_list;

/* main application Widget */
GtkWidget *app;

/* callback for the application */
void keypress_cb (GtkWidget *widget, GdkEventKey *event);

/* callback for the File menu */
void about_cb (GtkWidget *widget, void *data);
void quit_cb (GtkWidget *widget, void *data); /* use this "cb" for leaving */
void preference_cb (GtkWidget *widget, void *data);

/* callback for the New submenu */
void new_folder_cb (GtkWidget *widget, void *data);
void new_file_cb   (GtkWidget *widget, void *data);

/* callback for the Edition menu */
void copy_cb  (GtkWidget *widget, void *data);
void cut_cb   (GtkWidget *widget, void *data);
void paste_cb (GtkWidget *widget, void *data);
void delete_cb (GtkWidget *widget, void *data);

/* create the application */
void prepare_app();

/*--------------------- menu bar ---------------------------*/
GnomeUIInfo new_menu[] = {
  { GNOME_APP_UI_ITEM, _("Directory..."), NULL, new_folder_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_ITEM, _("File..."), NULL, new_file_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_ENDOFINFO }
};

GnomeUIInfo file_menu[] = {
  { GNOME_APP_UI_SUBTREE, _("New"), NULL, new_menu, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_SEPARATOR, NULL, NULL, NULL, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_ITEM, _("Preferences..."), NULL, preference_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PREF, 'P',
    GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_SEPARATOR, NULL, NULL, NULL, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_ITEM, _("Exit"), NULL, quit_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'Q',
    GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ENDOFINFO }
};

GnomeUIInfo edition_menu[] = {
  { GNOME_APP_UI_ITEM, _("Copy"), NULL, copy_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_COPY, 'C',
    GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, _("Cut"), NULL, cut_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CUT, 'X',
    GDK_CONTROL_MASK, NULL },    
  { GNOME_APP_UI_ITEM, _("Paste"), NULL, paste_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PASTE, 'V',
    GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, _("Delete"), NULL, delete_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TRASH, 0,
    GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ENDOFINFO }
};

GnomeUIInfo help_menu[] = {
    GNOMEUIINFO_HELP("gfile"),
  { GNOME_APP_UI_SEPARATOR, NULL, NULL, NULL, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_ITEM, _("About..."), NULL, about_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0, NULL },
  { GNOME_APP_UI_ENDOFINFO }
};

GnomeUIInfo main_menu[] = {
  { GNOME_APP_UI_SUBTREE, _("_File"), NULL, file_menu, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_SUBTREE, _("_Edit"), NULL, edition_menu, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
/*  { GNOME_APP_UI_JUSTIFY_RIGHT, NULL, NULL, NULL, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },*/
  { GNOME_APP_UI_SUBTREE, _("_Help"), NULL, help_menu, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_ENDOFINFO }
};
/*---------------------- menu bar end ------------------------------*/

/***************************************************/
/* Load last preferences written on the hard drive */
void gfile_load_preference(void)
{
  /* pour charger les anciennes dispositions */
/*  gchar *tmppath;
  gint file_handle;
*/
  /* charge les preferences */
/*  tmppath = (gchar *)g_malloc(strlen(GFILEDIR)+9);
  if(tmppath == NULL) { g_print("Can't allocate memory."); exit(1); }
*/  /*strcpy(tmppath,GFILEDIR);
  strcat(tmppath,"/aspect");
  file_handle = open(tmppath,O_RDONLY);
  if(file_handle != -1)
  { read(file_handle,&gfile_pref,sizeof(pref));
    close(file_handle);
  }*/
  gfile_preference.general_mode = MODE_TREE_LIST;
  gfiletreepref_copy(&gfile_preference.TreePreference,&DefaultGFileTreePref);
  gfile_preference.TreePreference.file_type_list = in_use_file_type_list;
  gfilelistpref_copy(&gfile_preference.LeftListPreference ,&DefaultGFileListPref);
  gfile_preference.LeftListPreference.owner_list = in_use_owner_list;
  gfile_preference.LeftListPreference.group_list = in_use_group_list;
  gfile_preference.LeftListPreference.file_type_list = in_use_file_type_list;
  gfilelistpref_copy(&gfile_preference.RightListPreference,&DefaultGFileListPref);
  gfile_preference.RightListPreference.owner_list = in_use_owner_list;
  gfile_preference.RightListPreference.group_list = in_use_group_list;
  gfile_preference.RightListPreference.file_type_list = in_use_file_type_list;
}
/***************************************************/

/******************************/
/* fill the GFILEDIR variable */
void gfile_init_base_directory(void)
{ gchar *tmp_string;
  struct stat *stat_buf;

  /* find the base directory */
    /* if GFILEDIR env variable is define, we take it */
  if(g_getenv("GFILEDIR") != NULL)
  {
      GFILEDIR = g_malloc(strlen(g_getenv("GFILEDIR"))+2);
      strcpy(GFILEDIR,g_getenv("GFILEDIR"));
      /* if not ended by a '/' add it */
      if((strlen(GFILEDIR)>0) && (GFILEDIR[strlen(GFILEDIR)-1] != '/'))
        strcat(GFILEDIR,"/");
    /* else we try to found in HOME/.GFile */
  } else {
     if(g_get_home_dir() == NULL)
     {gfile_error(CREATE_GFILEDIR); exit(CREATE_GFILEDIR);}
     GFILEDIR = g_malloc(strlen(g_get_home_dir())+9);
     strcpy(GFILEDIR,g_get_home_dir());
     if((strlen(GFILEDIR)>0) && (GFILEDIR[strlen(GFILEDIR)-1] != '/'))
       strcat(GFILEDIR,"/.GFile"); else strcat(GFILEDIR,".GFile");
     stat_buf = g_malloc(sizeof(struct stat));
     if(stat(GFILEDIR,stat_buf) == -1) /* if it does not exist */
     {
       g_print("No base directory define for GFile, HOME/.GFile will be made."\
               "\nYou can choose another by defining the environement"\
	       " variable GFILEDIR.\n"\
	       "ex : export GFILEDIR=/usr/local/gfile\n"\
	       "all icons MUST be place in GFILEDIR/%s\n",ICON_PATH);
       /* create the directory */
       if(mkdir(GFILEDIR, S_IRWXU) == -1)
       {gfile_error(CREATE_GFILEDIR); exit(CREATE_GFILEDIR);}
     }
     g_free(stat_buf);
     strcat(GFILEDIR,"/");
  }
  g_print("\nGFile base directory = %s \n",GFILEDIR);
}
/******************************/

/****************/
/* main program */
int main(int argc, char *argv[])
{ gchar *tmp_string;
  gchar *icon_path;
 
  gnome_init ("GFile", VERSION, argc, argv);

  /* fill GFILEDIR */ 
  gfile_init_base_directory();

  app = gnome_app_new ("GFile", "GFile");
  gtk_widget_realize (app);

  file_type_set_parent(app);

  icon_path = g_strconcat(GFILEDIR,ICON_PATH,"/",NULL);
  file_type_set_icon_path(icon_path);
  g_free(icon_path);
  
  /* Init the file type system use for the file type reconisation */
  tmp_string = g_strconcat(GFILEDIR,FILE_TYPE_FILE,NULL);
  in_use_file_type_list = file_type_list_load_from_file(tmp_string);
  if(in_use_file_type_list == NULL)
  {
    switch(gfile_errno)
    {
      case FILE_TYPE_LIST_FILE_TOO_BIG           :
        g_print("ERROR : file type list file \"%s\" is too big.\n",tmp_string);
        break;
      case FILE_TYPE_LIST_FILE_OPEN_ERROR        :
        g_print("ERROR : while opening file type list file \"%s\".\n",tmp_string);
        break;
      case FILE_TYPE_LIST_FILE_SYNTAXE_ERROR     :
        g_print("ERROR : syntaxe error in file type list file \"%s\".\n",tmp_string);
        break;
      case FILE_TYPE_LIST_FILE_INVALID_FILE_TYPE : 
        g_print("ERROR : invalid file type in file type list file \"%s\".\n",tmp_string);
        break;
      case FILE_TYPE_LIST_FILE_ICON_NOT_FOUND    :
        g_print("ERROR : invalid icon in file type list file \"%s\".",tmp_string);
        break;
    }
    exit(LOADING_FILE_TYPE_LIST);
  }
  file_type_list_add_needed_file_type(in_use_file_type_list);
  g_free(tmp_string);

  /* load information on users and groups */
  in_use_group_list = group_load();
  if(in_use_group_list == NULL){gfile_error(CREATE_GROUP_LIST); exit(CREATE_GROUP_LIST);} 
  in_use_owner_list = owner_load(in_use_group_list);
  if(in_use_owner_list == NULL){gfile_error(CREATE_OWNER_LIST); exit(CREATE_OWNER_LIST);}

  /* create a send to commands list */
  tmp_string = g_strconcat(GFILEDIR,SEND_TO_FILE,NULL);
  in_use_send_to_list = send_to_list_load_from_file(tmp_string);
  if(in_use_send_to_list == NULL)
  {
    switch(gfile_errno)
    {
      case SEND_TO_LIST_FILE_TOO_BIG           :
        g_print("ERROR : send to list file \"%s\" is too big.\n",tmp_string);
        break;
      case SEND_TO_LIST_FILE_OPEN_ERROR        :
        g_print("ERROR : while opening send to list file \"%s\".\n",tmp_string);
        break;
    }
    exit(LOADING_SEND_TO_LIST);
  }
  g_free(tmp_string);

  /* load preferences */
  gfile_load_preference();

  /* build the application */
  prepare_app ();

  /* load the file which define the look (if exist) */
  tmp_string = g_strconcat(GFILEDIR,RC_FILE,NULL);
  gtk_rc_parse(tmp_string);
  g_free(tmp_string);

  gtk_main();

  return(0);
}
/****************/

/* For the right part (LIST)(only use in MODE_TREE_LIST)*/
void right_down_dir_cb(GtkWidget *widget, void *data)
{
  /*g_print("right down directory\n");*/
  gfiletree_down(GFILETREE(LeftPart));
} 

/* For the right part (LIST)(only use in MODE_TREE_LIST)*/
void right_up_dir_cb(GtkWidget *widget, gchar *directory)
{
  /*g_print("right up to directory : %s\n",directory);*/
  gfiletree_up(GFILETREE(LeftPart), directory);
} 

/* For the left part (TREE)(only use in MODE_TREE_LIST) */
void left_change_dir_cb(GtkWidget *widget, gchar *directory)
{
/*  g_print("Tree move to directory : %s\n",directory);*/
  if(gfilelist_open_dir(GFILELIST(RightPart),directory) == FALSE)
  {
    g_print("the directory doesn't exist\n"); 
    /* __________ NOT FINISH, WE MUST GO TO THE PARENT DIRECTORY AND
      SUPPRESS THIS DIRECTORY ___________________________________ */
  }
}

/* signal when the right part scan an entire dir */
void right_scan_dir_cb(GtkWidget *widget, FileDataList *pFileDataList,gpointer data)
{
  /*g_print("right scan directory\n");*/
  switch(gfile_preference.general_mode)
  {
    case MODE_LIST_LIST :
      if(!strcmp(GFILELIST(LeftPart)->current_path,
           GFILELIST(RightPart)->current_path))
      { /*g_print("Left list will be update\n");*/
        gfilelist_update(GFILELIST(LeftPart), pFileDataList);
      }
      break;
    case MODE_TREE_LIST :
      gfiletree_update(GFILETREE(LeftPart), pFileDataList);
      break;
    default :
      g_print("ERROR : right scan dir signal\n");
  }
} 

/* signal when the left part scan an entire dir */
void left_scan_dir_cb(GtkWidget *widget, FileDataList *pFileDataList,gpointer data)
{
  /*g_print("left scan directory\n");*/
  switch(gfile_preference.general_mode)
  {
    case MODE_LIST_LIST :
      if(!strcmp(GFILELIST(LeftPart)->current_path,
           GFILELIST(RightPart)->current_path))
      { /*g_print("Right list will be update\n");*/
        gfilelist_update(GFILELIST(RightPart), pFileDataList);
      }
      break;
    case MODE_TREE_LIST :
      gfiletree_update(GFILETREE(RightPart), pFileDataList);
      break;
    default :
      g_print("ERROR : left scan dir signal\n");
  }
}


void LeftPart_active_cb(GtkWidget *widget,void *data)
{
  if(active_part == RIGHT_PART)
  {
    /*g_print("Left part selected\n");*/
    active_part = LEFT_PART;
    /* unselect the right part cause we are in the left part */
    /* what ever the general mode is, RightPart is ALWAYS a list */
    gfilelist_unselect_all(GFILELIST(RightPart));
  }  
}

void RightPart_active_cb(GtkWidget *widget,void *data)
{
  if(active_part == LEFT_PART)
  {
    /*g_print("Right part selected\n");*/
    active_part = RIGHT_PART;
    /* unselect the left part cause we are in the right part */
    /* LeftPart depend on the choosen general mode */
    if(gfile_preference.general_mode == MODE_LIST_LIST)
      gfilelist_unselect_all(GFILELIST(LeftPart));
    else  
      gfiletree_unselect_all(GFILETREE(LeftPart));
  }  
}

/* called eached time a file changed (name, size, rights ...) */
void file_mode_changed_cb(GtkWidget *Widget,FileData *SrcFile,
  FileData *DstFile,gpointer data)
{ gchar *str_SrcFile,*str_DstFile;

  switch(gfile_preference.general_mode)
  {
    case MODE_LIST :
      gfilelist_modify_file(GFILELIST(RightPart),SrcFile,DstFile);
      break;
    case MODE_TREE_LIST :
      str_SrcFile = g_strconcat(SrcFile->path,SrcFile->name,NULL);
      str_DstFile = g_strconcat(DstFile->path,DstFile->name,NULL); 
      gfiletree_modify_dir_by_name(GFILETREE(LeftPart),str_SrcFile,str_DstFile);
      g_free(str_SrcFile); g_free(str_DstFile);
      gfilelist_modify_file(GFILELIST(RightPart),SrcFile,DstFile);    
      break;    
    case MODE_LIST_LIST :        
      gfilelist_modify_file(GFILELIST(LeftPart),SrcFile,DstFile);    
      gfilelist_modify_file(GFILELIST(RightPart),SrcFile,DstFile);
      break;    
  }  
}  

void prepare_app()
{
  GtkMenuFactory *mf;
  
  gtk_signal_connect (GTK_OBJECT (app), "delete_event",
                      GTK_SIGNAL_FUNC (quit_cb),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (app), "key_press_event",
                      GTK_SIGNAL_FUNC (keypress_cb),
                      NULL);

  /* create the menu */
  gnome_app_create_menus ( GNOME_APP (app), main_menu);
  vbox = gtk_vbox_new(0,0);
  gtk_widget_show(vbox);
  gnome_app_set_contents(GNOME_APP (app), vbox);

  switch(gfile_preference.general_mode)
  {
    case MODE_LIST :
      /* create a list */
      RightPart = gfilelist_new(gfile_preference.RightListPreference);
      gtk_signal_connect (GTK_OBJECT(RightPart), "file_mode_changed",
        GTK_SIGNAL_FUNC (file_mode_changed_cb), NULL);
      
/*      gtk_signal_connect (GTK_OBJECT(RightPart), "down_dir",
        GTK_SIGNAL_FUNC (right_down_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(RightPart), "up_dir",
        GTK_SIGNAL_FUNC (right_up_dir_cb), NULL);*/
      /* don't need to know where we are if there is only a list */	

      gtk_widget_show (GTK_WIDGET(RightPart));
      gfilelist_open_dir(GFILELIST(RightPart),"/");
   
      gtk_box_pack_start_defaults(GTK_BOX(vbox),RightPart);
      
      active_part = RIGHT_PART; /* always RIGHT_PART */
      break;
    case MODE_TREE_LIST :
      /* create the vertical division */        
      hpaned = gtk_hpaned_new ();
      gtk_box_pack_start_defaults(GTK_BOX(vbox),hpaned);      
      gtk_widget_show (hpaned);

      /* create a gfiletree in the left part */
      LeftPart = gfiletree_new(gfile_preference.TreePreference);
      gtk_signal_connect (GTK_OBJECT(LeftPart), "change_dir",
        GTK_SIGNAL_FUNC (left_change_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(((GFileTree *)LeftPart)->Ctree),"tree_select_row",
        GTK_SIGNAL_FUNC(LeftPart_active_cb), NULL);
		      
      gtk_widget_show (GTK_WIDGET(LeftPart));
      gtk_paned_add1 (GTK_PANED(hpaned), GTK_WIDGET(LeftPart));

      /* create a list in the right part */
      RightPart = gfilelist_new(gfile_preference.RightListPreference);
      gtk_signal_connect (GTK_OBJECT(RightPart), "file_mode_changed",
        GTK_SIGNAL_FUNC (file_mode_changed_cb), NULL);
 
      gtk_signal_connect (GTK_OBJECT(RightPart), "down_dir",
        GTK_SIGNAL_FUNC (right_down_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(RightPart), "up_dir",
        GTK_SIGNAL_FUNC (right_up_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(RightPart), "scan_dir",
        GTK_SIGNAL_FUNC (right_scan_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(((GFileList *)RightPart)->Clist),"select_row",
        GTK_SIGNAL_FUNC(RightPart_active_cb), NULL);

      gtk_widget_show (GTK_WIDGET(RightPart));
      gfilelist_open_dir(GFILELIST(RightPart),"/");
      
      gtk_paned_add2 (GTK_PANED(hpaned), GTK_WIDGET(RightPart) );

      break;
    case MODE_LIST_LIST :
      /* create the vertical division */        
      hpaned = gtk_hpaned_new ();
      gtk_box_pack_start_defaults(GTK_BOX(vbox),hpaned);
      gtk_widget_show (hpaned);

      /* create a list in the left part */
      LeftPart = gfilelist_new(gfile_preference.LeftListPreference);
      gtk_signal_connect (GTK_OBJECT(LeftPart), "file_mode_changed",
        GTK_SIGNAL_FUNC (file_mode_changed_cb), NULL);
      
/*      gtk_signal_connect (GTK_OBJECT(LeftPart), "down_dir",
        GTK_SIGNAL_FUNC (left_down_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(LeftPart), "up_dir",
        GTK_SIGNAL_FUNC (left_up_dir_cb), NULL);*/
      gtk_signal_connect (GTK_OBJECT(LeftPart), "scan_dir",
        GTK_SIGNAL_FUNC (left_scan_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(((GFileList *)LeftPart)->Clist),"select_row",
        GTK_SIGNAL_FUNC(LeftPart_active_cb), NULL);

      gtk_widget_show (GTK_WIDGET(LeftPart));
      gfilelist_open_dir(GFILELIST(LeftPart),"/");
      
      gtk_paned_add1 (GTK_PANED(hpaned), GTK_WIDGET(LeftPart) );

      /* create a list in the right part */
      RightPart = gfilelist_new(gfile_preference.RightListPreference);
      gtk_signal_connect (GTK_OBJECT(RightPart), "file_mode_changed",
        GTK_SIGNAL_FUNC (file_mode_changed_cb), NULL);
      
/*      gtk_signal_connect (GTK_OBJECT(RightPart), "down_dir",
        GTK_SIGNAL_FUNC (right_down_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(RightPart), "up_dir",
        GTK_SIGNAL_FUNC (right_up_dir_cb), NULL);*/
      gtk_signal_connect (GTK_OBJECT(RightPart), "scan_dir",
        GTK_SIGNAL_FUNC (right_scan_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(((GFileList *)RightPart)->Clist),"select_row",
        GTK_SIGNAL_FUNC(RightPart_active_cb), NULL);

      gtk_widget_show (GTK_WIDGET(RightPart));
      gfilelist_open_dir(GFILELIST(RightPart),"/");
      
      gtk_paned_add2 (GTK_PANED(hpaned), GTK_WIDGET(RightPart) );
    
      break;    
  }      
  gtk_widget_show (app);
}

/* Callbacks functions */

/* This is called eached time a key is pressed */
void keypress_cb (GtkWidget *widget, GdkEventKey *event)
{
  /*g_print("Key pressed : %d\n",event->keyval);*/
  switch(event->keyval)
  {
     case 65307 : /* Esc = quit */
                  quit_cb(widget,event);
     		  break;
     case 65535 : /* Del = delete all the selected files */
     		  g_print("Del pressed\n");
		  delete_cb(NULL,NULL);
     		  break;
  }
}

/* callback du menu File */
void new_dir_cb (MkdirDialog *pMkdirDialog,gchar *NewFile, gpointer data)
{
  switch(gfile_preference.general_mode)
  {
    case MODE_LIST :
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);
      break;
    case MODE_LIST_LIST :
      gfilelist_add_file_by_full_name(GFILELIST(LeftPart),NewFile);
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);      
      break;
    case MODE_TREE_LIST :
      gfiletree_add_dir_by_name(GFILETREE(LeftPart),NewFile);
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);    
      break;      
  }
}

void new_folder_cb (GtkWidget *widget, void *data)
{ GtkWidget *vMkdirDialog;
  gchar *vpath;

  switch(active_part)
  { 
    case LEFT_PART :
      if(gfile_preference.general_mode == MODE_LIST_LIST)
        vpath = GFILELIST(LeftPart)->current_path;
      else
        vpath = GFILETREE(LeftPart)->current_path;
      break;
    case RIGHT_PART :
      vpath = GFILELIST(RightPart)->current_path;
      break;  
  }

  vMkdirDialog = mkdir_dialog_new(vpath);
  gtk_widget_show(vMkdirDialog);
  
  gtk_signal_connect (GTK_OBJECT(vMkdirDialog), "new_dir",
                      GTK_SIGNAL_FUNC (new_dir_cb),
                      NULL);
}

void new_file_cb (GtkWidget *widget, void *data)
{
/*  GtkWidget *window; */

  printf("new file called.\n");

/*  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "New file");
  gtk_signal_connect (GTK_OBJECT (window), "destroy",
  GTK_SIGNAL_FUNC (gtk_widget_destroyed), &window);

  gtk_widget_show (window);
*/
}

/**********************************************************/
/* change the general mode (choice are : MODE_TREE_LIST,  */ 
/* MODE_LIST_LIST and MODE_LIST)                          */
static void change_general_mode(guint8 new_general_mode)
{
  /* if not necessary, leave */
  if(gfile_preference.general_mode == new_general_mode) return;

  /* RightPart is separate from its parent and suppress */
  gtk_container_remove(GTK_CONTAINER(RightPart->parent), RightPart);

  /* create a list in the right part */
  RightPart = gfilelist_new(gfile_preference.RightListPreference);
  gfilelist_open_dir(GFILELIST(RightPart),"/");
  gtk_widget_show (GTK_WIDGET(RightPart));

  /* destroy the left part because we are sure that it is no more good */
  if((gfile_preference.general_mode == MODE_LIST_LIST) || 
     (gfile_preference.general_mode == MODE_TREE_LIST))
  {   
    /* LeftPart will be separate from its parent and remove */
    gtk_container_remove(GTK_CONTAINER(LeftPart->parent), LeftPart);
  }
  
  /* build the new left part */
  switch(new_general_mode)
  {
    case MODE_LIST_LIST :
      /* create a list in the left part */
      LeftPart = gfilelist_new(gfile_preference.LeftListPreference);
      gfilelist_open_dir(GFILELIST(LeftPart),"/");
      gtk_widget_show (GTK_WIDGET(LeftPart));
      gtk_signal_connect (GTK_OBJECT(LeftPart), "file_mode_changed",
        GTK_SIGNAL_FUNC (file_mode_changed_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(LeftPart), "scan_dir",
        GTK_SIGNAL_FUNC (left_scan_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(((GFileList *)LeftPart)->Clist),"select_row",
        GTK_SIGNAL_FUNC(LeftPart_active_cb), NULL);
      break;
    case MODE_TREE_LIST :
      /* create a gfiletree in the left part */
      LeftPart = gfiletree_new(gfile_preference.TreePreference);
      gtk_signal_connect (GTK_OBJECT(LeftPart), "change_dir",
        GTK_SIGNAL_FUNC (left_change_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(((GFileTree *)LeftPart)->Ctree),"tree_select_row",
        GTK_SIGNAL_FUNC(LeftPart_active_cb), NULL);
		      
      gtk_widget_show (GTK_WIDGET(LeftPart));
      /* right part restart to "/" to be in armony with the tree */
      gfilelist_open_dir(GFILELIST(RightPart),"/");
      break;      
  }
  
  /* if last mode was MODE_LIST, hpaned must be add */
  if(gfile_preference.general_mode == MODE_LIST)
  {
    /* create the vertical division */
    hpaned = gtk_hpaned_new ();
    gtk_widget_show (hpaned);
  } 
  
  if((new_general_mode == MODE_LIST_LIST) || 
     (new_general_mode == MODE_TREE_LIST))
  {   
    gtk_paned_add1 (GTK_PANED(hpaned), GTK_WIDGET(LeftPart) );
    gtk_paned_add2 (GTK_PANED(hpaned), GTK_WIDGET(RightPart) );
    if(gfile_preference.general_mode == MODE_LIST)
      gtk_box_pack_start_defaults(GTK_BOX(vbox),hpaned);
      
  } else {
    /* new_general_mode is MODE_LIST */
    /* hpaned is separate from its parent and suppress */
    gtk_container_remove(GTK_CONTAINER(hpaned->parent), hpaned);
    gtk_box_pack_start_defaults(GTK_BOX(vbox),RightPart);
  }

  /* signals for the RightPart (always a list) */  
  gtk_signal_connect (GTK_OBJECT(RightPart), "file_mode_changed",
    GTK_SIGNAL_FUNC (file_mode_changed_cb), NULL);  
  if(new_general_mode == MODE_TREE_LIST)
  {
    gtk_signal_connect (GTK_OBJECT(RightPart), "down_dir",
      GTK_SIGNAL_FUNC (right_down_dir_cb), NULL);
    gtk_signal_connect (GTK_OBJECT(RightPart), "up_dir",
      GTK_SIGNAL_FUNC (right_up_dir_cb), NULL);
  }
  if(new_general_mode != MODE_LIST)
  {
    gtk_signal_connect (GTK_OBJECT(((GFileList *)RightPart)->Clist),"select_row",
      GTK_SIGNAL_FUNC(RightPart_active_cb), NULL);
    gtk_signal_connect (GTK_OBJECT(RightPart), "scan_dir",
      GTK_SIGNAL_FUNC (right_scan_dir_cb), NULL);
  }

  gfile_preference.general_mode = new_general_mode;
}
/**********************************************************/

/* procedure executed when Apply is pressed in the preference dialog */
static void preference_dialog_apply_cb(PreferenceDialog *pPreferenceDialog,
  PreferenceDialogPref *pPreferenceDialogPref, void *data)
{
  g_print("Apply pressed\n");
  
  /* replace old file type list by the new one */
  file_type_list_free(in_use_file_type_list);
  /* copy the new one at the place of the old one */
  in_use_file_type_list = file_type_list_copy(pPreferenceDialogPref->file_type_list);

  gfile_preference.RightListPreference.file_type_list = in_use_file_type_list;
  gfile_preference.LeftListPreference.file_type_list  = in_use_file_type_list;
  gfile_preference.TreePreference.file_type_list      = in_use_file_type_list;


  /* change the file type list for tree and lists */
  if(gfile_preference.general_mode == MODE_TREE_LIST)
    gfiletree_set_filetype(GFILETREE(LeftPart),in_use_file_type_list);
  else if(gfile_preference.general_mode == MODE_LIST_LIST)
    gfilelist_set_filetype(GFILELIST(LeftPart),in_use_file_type_list);
  
  /* RightPart is always a list */
  gfilelist_set_filetype(GFILELIST(RightPart),in_use_file_type_list);

  /* replace old send to list by the new one */
  send_to_list_free(in_use_send_to_list);
  /* copy the new one at the place of the old one */
  in_use_send_to_list = send_to_list_copy(pPreferenceDialogPref->send_to_list);
  
  /* we change the general mode if needed */
  change_general_mode(pPreferenceDialogPref->general_mode);
  
  if(gfile_preference.LeftListPreference.sort_mode != pPreferenceDialogPref->left_list_sort_mode)
  {
    gfile_preference.LeftListPreference.sort_mode = pPreferenceDialogPref->left_list_sort_mode;
    if(gfile_preference.general_mode == MODE_LIST_LIST)
      gfilelist_set_sort(GFILELIST(LeftPart),gfile_preference.LeftListPreference.sort_mode);
  }
  if(gfile_preference.RightListPreference.sort_mode != pPreferenceDialogPref->right_list_sort_mode)
  {
    gfile_preference.RightListPreference.sort_mode = pPreferenceDialogPref->right_list_sort_mode;
    gfilelist_set_sort(GFILELIST(RightPart),gfile_preference.RightListPreference.sort_mode);
  }  
}

void preference_cb (GtkWidget *widget, void *data)
{ PreferenceDialogPref *vPreferenceDialogPref;
  GtkWidget *vPreferenceDialog;
  
  vPreferenceDialogPref = g_malloc(sizeof(PreferenceDialogPref));
  vPreferenceDialogPref->general_mode = gfile_preference.general_mode;
  /* replace this by the sort mode use now */
  if(gfile_preference.general_mode == MODE_LIST_LIST)
    vPreferenceDialogPref->left_list_sort_mode  = 
      gfilelist_get_sort(GFILELIST(LeftPart));
  else
    vPreferenceDialogPref->left_list_sort_mode  = 
      gfile_preference.LeftListPreference.sort_mode;
  /* replace this by the sort mode use now (right part is always a list)*/
  vPreferenceDialogPref->right_list_sort_mode = 
    gfilelist_get_sort(GFILELIST(RightPart));
    
  vPreferenceDialogPref->owner_list = in_use_owner_list;
  vPreferenceDialogPref->group_list = in_use_group_list;
  vPreferenceDialogPref->file_type_list = in_use_file_type_list;
  vPreferenceDialogPref->file_type_save_file = g_strconcat(GFILEDIR,FILE_TYPE_FILE,NULL);
  vPreferenceDialogPref->send_to_list = in_use_send_to_list;
  vPreferenceDialogPref->send_to_save_file = g_strconcat(GFILEDIR,SEND_TO_FILE,NULL);

  vPreferenceDialog = preference_dialog_new(*vPreferenceDialogPref);

  gtk_signal_connect(GTK_OBJECT(vPreferenceDialog),"apply",
    GTK_SIGNAL_FUNC(preference_dialog_apply_cb),NULL);
    
  gtk_widget_show(vPreferenceDialog);

  g_free(vPreferenceDialogPref->file_type_save_file);
  g_free(vPreferenceDialogPref->send_to_save_file);
  g_free(vPreferenceDialogPref);
  
/*  printf("preferences called\n");*/
}

void quit_cb (GtkWidget *widget, void *data)
{// gchar *tmppath;
//  gint file_handle;
//  guint16 taille[4];
//  gint col;
  
/*  tmppath = (gchar *)g_malloc(strlen(GFILEDIR)+8);*/
  
  /* on enregistre les infos de position */
/*  gfile_pref.ctree_width  = (GTK_WIDGET(ctree))->allocation.width;
  gfile_pref.ctree_height = (GTK_WIDGET(ctree))->allocation.height;
  gfile_pref.clist_width  = (GTK_WIDGET(clist))->allocation.width;
  gfile_pref.clist_height = (GTK_WIDGET(clist))->allocation.height;
  gfile_pref.nb_list_col  = ((GtkCList *)clist)->columns;*/
/*
  if(gfile_pref.nb_list_col > 0) 
    gfile_pref.col1_width = ((GtkCList *)clist)->column[0].width;
  if(gfile_pref.nb_list_col > 1) 
    gfile_pref.col2_width = ((GtkCList *)clist)->column[1].width;
  if(gfile_pref.nb_list_col > 2) 
    gfile_pref.col3_width = ((GtkCList *)clist)->column[2].width;
  if(gfile_pref.nb_list_col > 3) 
    gfile_pref.col4_width = ((GtkCList *)clist)->column[3].width;
  if(gfile_pref.nb_list_col > 4) 
    gfile_pref.col5_width = ((GtkCList *)clist)->column[4].width;
*/  
/*  g_print("ctree w : %u, h : %u\n",(GTK_WIDGET(ctree))->allocation.width,
          (GTK_WIDGET(ctree))->allocation.height); */
/*
  g_print("clist w : %u, h : %u\n",(GTK_WIDGET(clist))->allocation.width,
          (GTK_WIDGET(clist))->allocation.height); 

  for(col=0;col<((GtkCList *)clist)->columns;col++)
  {
    g_print("col %d width : %d\n",col,((GtkCList *)clist)->column[col].width);
  }
*/
/*  strcpy(tmppath,GFILEDIR);
  strcat(tmppath,"/aspect");
  file_handle = creat(tmppath,S_IREAD|S_IWRITE);
  if(file_handle != -1)
  {
    write(file_handle,&gfile_pref,sizeof(pref));
    close(file_handle);
  }
  g_free(tmppath);*/

  
  /* Free the file type system */  
  file_type_list_free(in_use_file_type_list);
  
  /* Free the users/groups */
  owner_free(in_use_owner_list);
  group_free(in_use_group_list);
  
  /* free the send to system */
  send_to_list_free(in_use_send_to_list);
  
  gtk_main_quit();
  return;
}

/* callback du menu Edition */
void copy_cb (GtkWidget *widget, void *data)
{
  if(copy_file_list != NULL){file_data_list_free(copy_file_list); copy_file_list = NULL;}
  switch(active_part)
  { 
    case LEFT_PART :
      if(gfile_preference.general_mode == MODE_LIST_LIST)
        copy_file_list = gfilelist_list_selected(GFILELIST(LeftPart));
      else
        copy_file_list = gfiletree_list_selected(GFILETREE(LeftPart));
      break;
    case RIGHT_PART :
      copy_file_list = gfilelist_list_selected(GFILELIST(RightPart));
      break;  
  }
  cut_mode = FALSE;
}

void cut_cb (GtkWidget *widget, void *data)
{
  if(copy_file_list != NULL){file_data_list_free(copy_file_list); copy_file_list = NULL;}
  switch(active_part)
  { 
    case LEFT_PART :
      if(gfile_preference.general_mode == MODE_LIST_LIST)
        copy_file_list = gfilelist_list_selected(GFILELIST(LeftPart));
      else
        copy_file_list = gfiletree_list_selected(GFILETREE(LeftPart));
      break;
    case RIGHT_PART :
      copy_file_list = gfilelist_list_selected(GFILELIST(RightPart));
      break;  
  }      
  cut_mode = TRUE;
}

/* transform an abolute name into a relative one */
void abs_2_rel(gchar *full_name,gchar **path,gchar **name)
{
  guint lastchar = 0;
  guint length = 0;
  guint16 compt;

  if(strcmp(full_name,"/")) /* if we are not a the root ("/") */
  {
     /* find */
     for(lastchar=0;full_name[lastchar];lastchar++);
     length = lastchar;
     lastchar-=2;
     for(;lastchar && (full_name[lastchar]!='/');lastchar--);
     g_print("length : %u, path length : %u.\n",length,lastchar);
     /* make a copy of the good part */
     *path = g_malloc(lastchar+2);
     for(compt=0;compt<=lastchar;compt++) (*path)[compt]=full_name[compt];
     (*path)[lastchar+1]=0;
     *name = g_malloc(length-lastchar);
     for(;(compt<length) && full_name[compt] != '/';compt++) (*name)[compt-lastchar-1]=full_name[compt];
     (*name)[compt-lastchar-1] = 0;
  } else 
  { *path = g_malloc(1); path[0] = 0;
    *name = g_malloc(2); strcpy(*name,"/");
  }
}

void file_added_cb (gchar *NewFile)
{
  g_print("GFILE : New file to add : %s.\n",NewFile);
  switch(gfile_preference.general_mode)
  {
    case MODE_LIST :
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);
      break;
    case MODE_LIST_LIST :
      gfilelist_add_file_by_full_name(GFILELIST(LeftPart),NewFile);
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);      
      break;
    case MODE_TREE_LIST :
      gfiletree_add_dir_by_name(GFILETREE(LeftPart),NewFile);
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);    
      break;      
  }
}

void file_move_cb (gchar *LastFile,gchar *NewFile)
{
  g_print("GFILE : File to move : %s.\n",LastFile);
  switch(gfile_preference.general_mode)
  {
    case MODE_LIST :
      gfilelist_remove_file_by_full_name(GFILELIST(RightPart),LastFile);
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);
      break;
    case MODE_LIST_LIST :
      gfilelist_remove_file_by_full_name(GFILELIST(LeftPart),LastFile);
      gfilelist_add_file_by_full_name(GFILELIST(LeftPart),NewFile);
      gfilelist_remove_file_by_full_name(GFILELIST(RightPart),LastFile);      
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);      
      break;
    case MODE_TREE_LIST :
      gfiletree_remove_dir_by_name(GFILETREE(LeftPart),LastFile);
      gfiletree_add_dir_by_name(GFILETREE(LeftPart),NewFile);
      gfilelist_remove_file_by_full_name(GFILELIST(RightPart),LastFile);
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);
      break;      
  }
}

void paste_cb (GtkWidget *widget, void *data)
{
  if(copy_file_list == NULL) return;
  /* cut */
  if(cut_mode == TRUE)
  {
    if(active_part == RIGHT_PART)
      move_dialog_new(copy_file_list,GFILELIST(RightPart)->current_path,file_move_cb);
    else {
      if(gfile_preference.general_mode == MODE_LIST_LIST)
        move_dialog_new(copy_file_list,GFILELIST(LeftPart)->current_path,file_move_cb);
      else 
        move_dialog_new(copy_file_list,GFILETREE(LeftPart)->current_path,file_move_cb);
    }  
  /* else copy */    
  } else {
    if(active_part == RIGHT_PART)
      copy_dialog_new(copy_file_list,GFILELIST(RightPart)->current_path,file_added_cb);
    else {
      if(gfile_preference.general_mode == MODE_LIST_LIST)
        copy_dialog_new(copy_file_list,GFILELIST(LeftPart)->current_path,file_added_cb);
      else 
        copy_dialog_new(copy_file_list,GFILETREE(LeftPart)->current_path,file_added_cb);    
    }  
  }
}

void file_del_func_cb(gchar *LastFile)
{
  g_print("GFILE : File to remove : %s.\n",LastFile);
  switch(gfile_preference.general_mode)
  {
    case MODE_LIST :
      gfilelist_remove_file_by_full_name(GFILELIST(RightPart),LastFile);    
      break;
    case MODE_LIST_LIST :
      gfilelist_remove_file_by_full_name(GFILELIST(LeftPart),LastFile);
      gfilelist_remove_file_by_full_name(GFILELIST(RightPart),LastFile);
      break;
    case MODE_TREE_LIST :
      gfiletree_remove_dir_by_name(GFILETREE(LeftPart),LastFile);
      gfilelist_remove_file_by_full_name(GFILELIST(RightPart),LastFile);
      break;      
  }
}

void delete_cb (GtkWidget *widget, void *data)
{ FileDataList *del_file_list;

  switch(active_part)
  { 
    case LEFT_PART :
      if(gfile_preference.general_mode == MODE_LIST_LIST)
        del_file_list = gfilelist_list_selected(GFILELIST(LeftPart));
      else
        del_file_list = gfiletree_list_selected(GFILETREE(LeftPart));
      break;
    case RIGHT_PART :
      del_file_list = gfilelist_list_selected(GFILELIST(RightPart));
      break;  
  }
  del_dialog_new(del_file_list,file_del_func_cb);
  
  file_data_list_free(del_file_list);
}

/* Help menu callback */
void about_cb (GtkWidget *widget, void *data)
{
  GtkWidget *about;
  const gchar *authors[] = {
	  "Daniel Lacroix",
          NULL
          };
  about = gnome_about_new ("GFile", VERSION,
        		/* copyright notice */
                        "(C) 1999, under the GNU General Public License",
                        authors,
                        /* another comments */
                        _("GFile is a File Manager"),
                        NULL);
  gtk_widget_show (about);
}
