/****************************************/
/*  GFile.c headers                     */
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#ifndef __GFILE_H__
#define __GFILE_H__

#include <gnome.h>    /* GTK/GNOME definitions            */
#include <dirent.h>   /* To list the directory file       */
#include <sys/stat.h> /* For the files properties         */
#include <fcntl.h>    /* for O_RONLY... use by FileType   */
#include <errno.h>
#include <sys/vfs.h>  /* for statfs */

#include "GFileError.h"       /* For all the error codes   */
#include "General.h"          /* some usefull functions    */

#include "FileType.h" 	      /* For managing file type    */
#include "FileTypeAutomate.h" /* For the FileType Automate */
			      /* ex: p.gif belong to *.gif */
#include "FileTypeList.h"     /* For FileType lists        */

#include "SendTo.h" 	      /* For managing send to commands */
#include "SendToList.h"       /* For SendTo lists          */

#include "PreferenceDialog.h" /* For the Preference dialog */
#include "PreferenceGeneral.h"/* For the General part of   */
                              /* the Preference dialog     */
#include "PreferenceList.h"   /* For the List part of      */
                              /* the Preference dialog     */
#include "PreferenceFileType.h"
			      /* for the File Type part of */
			      /* the Preference dialog     */
#include "PreferenceFileTypeAddEdit.h"
			      /* for the Add & Edit buttons*/
			      /* of the File Type part of  */
			      /* the Preference dialog     */
#include "PreferenceSendTo.h" /* for the Send To part of   */
			      /* the Preference dialog     */
#include "PreferenceSendToAddEdit.h"
			      /* for the Add & Edit buttons*/
			      /* of the Send to part of    */
			      /* the Preference dialog     */
#include "IconList.h"         /* for the icon button in the*/
			      /* Add & Edit dialog         */
#include "OwnerGroup.h"       /* For a list which give name*/
			      /* to number		   */
#include "FileData.h"         /*data which represent a file*/
#include "FileDataList.h"     /* List of FileData          */
#include "CompareFunc.h"      /* Compare functions for     */
                              /* gtk_clist & FileData      */
#include "GFileList.h"	      /* For the list part of GFile*/
#include "GFileTree.h"	      /* For the tree part of GFile*/
#include "CopyDialog.h"       /* For the file copy dialog  */
#include "DelDialog.h"        /* For the file del dialog   */
#include "MoveDialog.h"       /* For the file move dialog  */
#include "FileProperties.h"   /* For the file properties   */
                              /* dialog.                   */
#include "MkdirDialog.h"      /* Make directory dialog     */
#include "PopMenu.h"          /* For the popup menu        */

#define VERSION "0.0.7"

/*****************************************/
/* all preferences are in this structure */
typedef struct _Preference Preference;

struct _Preference
{
  /* MODE_TREE_LIST, MODE_LIST_LIST or MODE_LIST */
  guint8 general_mode;
  
  GFileTreePref TreePreference;
  
  GFileListPref LeftListPreference;
  /*when one list is selected, we take preferences of the Right List*/
  GFileListPref RightListPreference;  
};
/*****************************************/

/* GFILEDIR indicate where is the GFile base directory */
/* it is based on the environment variable GFILEDIR    */
/* This is ALWAYS ended by a '/'                       */
extern gchar *GFILEDIR;

/* for the right file list */
extern GtkWidget *RightPart;
/* For the left part (tree, list or nothing) */
extern GtkWidget *LeftPart;
/* List of all file in copy/cut mode */
extern FileDataList *copy_file_list;
extern gboolean cut_mode; /* if TRUE,we are in a cut else a copy */

/* all preferences = file "gfile_preference" */
extern Preference gfile_preference;

/* to know if the user is now working in the left or the right part */
/* like that, if copy, del... is called job will be done in the good*/
/* part                                                             */
enum {LEFT_PART, RIGHT_PART};
extern guint8 active_part;

/* list of all known file type */
extern FileTypeList *in_use_file_type_list;

/* list of all known group on this computer */
extern struct _Group *in_use_group_list;

/* list of all known users on this computer */
extern struct _Owner *in_use_owner_list;

/* list of all known send to commands */
extern SendToList *in_use_send_to_list;

void file_added_cb (gchar *NewFile);
void file_move_cb (gchar *LastFile,gchar *NewFile);
void file_del_func_cb(gchar *LastFile);
/* called eached time a file changed (name, size, rights ...) */
void file_mode_changed_cb(GtkWidget *Widget,FileData *SrcFile,
  FileData *DstFile,gpointer data);

#endif  /* __GFILE_H__ */
