/****************************************/
/*  MkdirDialog.c body           	*/
/*					*/
/*  Description : Manage Mkdir (create a*/
/*                new directory) widget	*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

static void mkdir_dialog_class_init(MkdirDialogClass *pMkdirDialogClass);
static void mkdir_dialog_init(MkdirDialog *pMkdirDialog);
static void destroy_cb(GtkWidget *Widget, MkdirDialog *pMkdirDialog);
static void cancel_cb(GtkWidget *Widget, MkdirDialog *pMkdirDialog);
static void ok_cb(GtkWidget *Widget, MkdirDialog *pMkdirDialog);

/*************************************************************/
guint mkdir_dialog_get_type ()
{
  static guint mkdir_dialog_type = 0;

  if (!mkdir_dialog_type)
  {
    GtkTypeInfo mkdir_dialog_info =
    {
       "MkdirDialog",
       sizeof (MkdirDialog),
       sizeof (MkdirDialogClass),
       (GtkClassInitFunc)  mkdir_dialog_class_init,
       (GtkObjectInitFunc) mkdir_dialog_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     mkdir_dialog_type = gtk_type_unique(gnome_dialog_get_type(), &mkdir_dialog_info);
  }
  return mkdir_dialog_type;
}
/*************************************************************/

/*************************************************************/
enum { NEW_DIR_SIGNAL, LAST_SIGNAL};
static gint mkdir_dialog_signals[LAST_SIGNAL] = { 0 };

static void string_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*string_marshal_func)(MkdirDialog *, gchar *,gpointer);
    string_marshal_func rfunc = (string_marshal_func) func;
    (*rfunc) (MKDIR_DIALOG(object),
	      GTK_VALUE_POINTER(args[0]), func_data);
}

static void mkdir_dialog_class_init(MkdirDialogClass *class)
{
  /* here we create the new signals if we want */
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) class;

  mkdir_dialog_signals[NEW_DIR_SIGNAL] = gtk_signal_new ("new_dir",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (MkdirDialogClass,new_dir),
            string_marshal, GTK_TYPE_NONE, 1,GTK_TYPE_POINTER);

   gtk_object_class_add_signals (object_class, mkdir_dialog_signals, LAST_SIGNAL);
   class->new_dir = NULL;
}
/*************************************************************/

static gchar *current_DestPath = NULL;

/*************************************************************/
static void mkdir_dialog_init(MkdirDialog *pMkdirDialog)
{ GtkWidget *label;
  GtkWidget *hbox;
  gchar *string;

  pMkdirDialog->DestPath = current_DestPath;

  gtk_window_set_title(GTK_WINDOW(pMkdirDialog),"Make directory");
  /* add the OK and CANCEL button */
  gnome_dialog_append_buttons(GNOME_DIALOG(pMkdirDialog),
    GNOME_STOCK_BUTTON_OK,GNOME_STOCK_BUTTON_CANCEL,NULL);

  gtk_signal_connect(GTK_OBJECT(pMkdirDialog), "destroy",
    GTK_SIGNAL_FUNC(destroy_cb),pMkdirDialog); 

  /* button Ok */
  gnome_dialog_button_connect(GNOME_DIALOG(pMkdirDialog),0,
    GTK_SIGNAL_FUNC(ok_cb),pMkdirDialog);  

  /* button Cancel */
  gnome_dialog_button_connect(GNOME_DIALOG(pMkdirDialog),1,
    GTK_SIGNAL_FUNC(cancel_cb),pMkdirDialog);  

  string = g_strconcat("path : ",pMkdirDialog->DestPath,NULL);
  label = gtk_label_new(pMkdirDialog->DestPath);
  gtk_widget_show(label);

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pMkdirDialog)->vbox),
    label,TRUE, TRUE, 2); 
    
  hbox = gtk_hbox_new(FALSE,2);
  gtk_widget_show(hbox);

  label = gtk_label_new("Name");
  gtk_widget_show(label);

  gtk_box_pack_start(GTK_BOX(hbox),
    label,TRUE, TRUE, 2); 
  
  pMkdirDialog->name = gtk_entry_new();
  gtk_widget_show(pMkdirDialog->name);

  gtk_box_pack_start(GTK_BOX(hbox),
    pMkdirDialog->name,TRUE, TRUE, 2); 
  
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pMkdirDialog)->vbox),
    hbox,TRUE, TRUE, 2); 
  
}
/*************************************************************/

/***************************************************/
/* called when the user click on the ok button     */
static void ok_cb(GtkWidget *Widget, MkdirDialog *pMkdirDialog)
{ gchar *full_name;
  gchar *name;
  gchar *name_test;
  gchar *error_msg;
  GtkWidget *error_dialog;
  /* NOT DONE */
  name = gtk_entry_get_text(GTK_ENTRY(pMkdirDialog->name));
  if(name[0]==0) return;
  name_test = name;
  while(*name_test){if(*name_test == '/') return; name_test++;}
  full_name = g_strconcat(pMkdirDialog->DestPath,name,NULL);

  if(mkdir(full_name,S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) == -1)
  {
    switch(errno)
    {
       case EEXIST : 
         error_msg = "pathname already exists (not\n"\
	             "necessarily as a directory)";
         break;
       case EFAULT : 
         error_msg = "pathname  points  outside  your accessible\n"\
	             "address space";
         break;
       case EACCES : 
         error_msg = "The parent directory does not allow write permission\n"\
                     "to the process, or one of the directories in\n"\
                     "pathname did not allow search (execute) permission";
         break;
       case ENAMETOOLONG :
         error_msg = "pathname was too long";
         break;
       case ENOENT :
         error_msg =  "A directory component in pathname does\n"\
                      "not exist or is a dangling symbolic link";
         break;
       case ENOTDIR : 
         error_msg = "A component used as a directory in pathname\n"\
                     "is not, in fact, a directory";
         break;
       case ENOMEM : 
         error_msg = "Insufficient kernel memory\n"\
	             "was available";
         break;
       case EROFS : 
         error_msg = "pathname refers to a file on\n"\
	             "a read-only filesystem";
         break;
       case ELOOP : 
         error_msg = "Too many symbolic links were encountered\n"\
	             "in resolving pathname";
         break;
       case ENOSPC : 
         error_msg = "The device containing pathname has\n"\
	             "no room for the new directory";
         break;
       default : /* I don't known the exact error num */
         error_msg = "The new directory cannot be created because\n"\
                     "your disk quota is exhausted";
         break;
/*       default :
         error_msg = "Unknown ERROR (BUG?)";
         break;*/
    }
    error_dialog = gnome_message_box_new(error_msg,GNOME_MESSAGE_BOX_ERROR,
      GNOME_STOCK_BUTTON_OK,NULL);
    gtk_grab_add(error_dialog);
    gtk_signal_connect(GTK_OBJECT(error_dialog),"destroy",
      GTK_SIGNAL_FUNC(gtk_grab_remove),NULL);
    gtk_widget_show(error_dialog);
  } else {
    gtk_signal_emit(GTK_OBJECT(pMkdirDialog),mkdir_dialog_signals[NEW_DIR_SIGNAL],
      full_name);
  }
  g_free(full_name);
  gnome_dialog_close (GNOME_DIALOG(pMkdirDialog)); 
}
/***************************************************/

/***************************************************/
/* called when the user click on the cancel button */
static void cancel_cb(GtkWidget *Widget, MkdirDialog *pMkdirDialog)
{ gnome_dialog_close (GNOME_DIALOG(pMkdirDialog)); }
/***************************************************/

/***************************************/
/* close when the dialog box is closed */
static void destroy_cb(GtkWidget *Widget, MkdirDialog *pMkdirDialog)
{
  g_free(pMkdirDialog->DestPath);
}
/***************************************/

GtkWidget *mkdir_dialog_new (gchar *DestPath)
{
  if(DestPath == NULL) return(NULL);
  current_DestPath = g_strdup(DestPath);
  return( GTK_WIDGET(gtk_type_new( mkdir_dialog_get_type() )) );
}
