/******************************************/
/*  PopMenu.c body         	          */
/*					  */
/*  Description : Popup menu which appear */ 
/*                when right clicking     */
/*                on some selected file   */
/*					  */
/*  Written by : Daniel Lacroix	          */
/*  GFile is under the GPL2 licence       */
/******************************************/

#include "GFile.h"

/* possible because there is only one popup at a time */
/* use as a parameter for cb                          */
FileDataList *last_FileDataList = NULL;

void pop_copy_cb(GtkWidget *widget, FileDataList *pFileDataList)
{
  g_print("pop_copy_cb\n");
  if(copy_file_list != NULL){file_data_list_free(copy_file_list); copy_file_list = NULL;}
  copy_file_list = file_data_list_copy(pFileDataList);
  cut_mode = FALSE;
  file_data_list_init(pFileDataList);
  while(file_data_list_is_end(pFileDataList))
  {
    g_print("c> %s\n",file_data_list_current(pFileDataList)->name);
    file_data_list_next(pFileDataList);
  }
}

void pop_cut_cb (GtkWidget *widget, FileDataList *pFileDataList)
{
  g_print("pop_cut_cb\n");
  if(copy_file_list != NULL){file_data_list_free(copy_file_list); copy_file_list = NULL;}
  copy_file_list = file_data_list_copy(pFileDataList);
  cut_mode = TRUE;
}

void pop_paste_cb(GtkWidget *widget, gchar *path)
{
  g_print("pop_paste_cb\n");
  if(copy_file_list == NULL) return;
  /* cut */
  if(cut_mode == TRUE)
    move_dialog_new(copy_file_list,path,file_move_cb);
  /* else copy */    
  else
    copy_dialog_new(copy_file_list,path,file_added_cb);
}

void pop_del_cb(GtkWidget *widget, FileDataList *pFileDataList)
{ FileDataList *del_file_list;

  del_file_list = file_data_list_copy(pFileDataList);
  del_dialog_new(del_file_list,file_del_func_cb);
  
  file_data_list_free(del_file_list);
}

void pop_prop_cb(GtkWidget *widget, FileDataList *pFileDataList)
{
  GtkWidget *propreties;
  
  propreties = file_properties_new(pFileDataList, in_use_file_type_list,
    in_use_owner_list, in_use_group_list);
  gtk_widget_show(propreties);
  
  gtk_signal_connect(GTK_OBJECT(propreties),"file_mode_changed",
    GTK_SIGNAL_FUNC(file_mode_changed_cb),NULL);
}

void pop_send_to_cb(GtkWidget *widget,SendTo *pSendTo)
{ gchar *string1,*string2;

  string1 = g_strdup(send_to_command(pSendTo));

  file_data_list_init(last_FileDataList);
  while(file_data_list_is_end(last_FileDataList) == FALSE)
  {
    string2 = string1;
    string1 = g_strdup_printf("%s \"%s%s\"",string1,
      file_data_list_current(last_FileDataList)->path,
      file_data_list_current(last_FileDataList)->name);
    g_free(string2);
    
    file_data_list_next(last_FileDataList);
  }
  system(string1);
  g_free(string1);
}

void pop_menu_new(FileDataList *pFileDataList, gchar *path, guint button,
  guint32 activate_time)
{ GtkWidget *pop_menu,*menu_items;
  static gchar *last_path = NULL;
  GtkWidget *send_to_pop;

  if(path == NULL) return;
  /* to avoid menu when user is on / of the tree */
  if(pFileDataList != NULL)
  {
    file_data_list_init(pFileDataList);
    if(file_data_list_is_end(pFileDataList) == FALSE)
    {
      file_data_list_next(pFileDataList);
      if(file_data_list_is_end(pFileDataList) == TRUE)
      {
        file_data_list_init(pFileDataList);
        if(!strcmp(file_data_list_current(pFileDataList)->path,"")) return;
      }
    }
  }

  pop_menu = gtk_menu_new();

  /* if no file data list then exit */
  if(pFileDataList == NULL) goto NONE_SELECTED;

  if(last_FileDataList != NULL) file_data_list_free(last_FileDataList);
  last_FileDataList = file_data_list_copy(pFileDataList);

  if(path != NULL) g_free(last_path);
  last_path = g_strdup(path);

  file_data_list_init(pFileDataList);

  /* if the file data list is empty */
  if(file_data_list_is_end(pFileDataList) == TRUE)
  {
    NONE_SELECTED :
    /* TO DO THE NEW SUB MENU */
    /*menu_items = gtk_menu_item_new_with_label("New");
    gtk_widget_set_sensitive(menu_items,FALSE);
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);*/

    /* insert a line */
    /*menu_items = gtk_menu_item_new();
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);*/

    menu_items = gtk_menu_item_new_with_label("Paste");
    gtk_signal_connect(GTK_OBJECT(menu_items), "activate",
      GTK_SIGNAL_FUNC(pop_paste_cb), (gpointer)last_path);
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);

  } else {

  file_data_list_next(pFileDataList);

  /* case when there is only one file */
  if(file_data_list_is_end(pFileDataList) == TRUE)
  {
    /* TO DO open or exec if a file */
    /*menu_items = gtk_menu_item_new_with_label("Open");
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);*/

    /* insert a line */
    /*menu_items = gtk_menu_item_new();
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);*/

    menu_items = gtk_menu_item_new_with_label("Copy");
    gtk_signal_connect(GTK_OBJECT(menu_items), "activate",
      GTK_SIGNAL_FUNC(pop_copy_cb), (gpointer)last_FileDataList);
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);  

    menu_items = gtk_menu_item_new_with_label("Cut");
    gtk_signal_connect(GTK_OBJECT(menu_items), "activate",
      GTK_SIGNAL_FUNC(pop_cut_cb), (gpointer)last_FileDataList);
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);

    menu_items = gtk_menu_item_new_with_label("Delete");
    gtk_signal_connect(GTK_OBJECT(menu_items), "activate",
      GTK_SIGNAL_FUNC(pop_del_cb), (gpointer)last_FileDataList);
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);

    /* insert a line */
    menu_items = gtk_menu_item_new();
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);
    
    /* for the send to sub menu */
    menu_items = gtk_menu_item_new_with_label("Send to");
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);
    send_to_pop = gtk_menu_new();
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_items),send_to_pop);
    send_to_list_init(in_use_send_to_list);
    while(send_to_list_is_end(in_use_send_to_list) == FALSE)
    {
      menu_items = gtk_menu_item_new_with_label(send_to_command(send_to_list_current(in_use_send_to_list)));
      gtk_signal_connect(GTK_OBJECT(menu_items), "activate",
        GTK_SIGNAL_FUNC(pop_send_to_cb), (gpointer)send_to_list_current(in_use_send_to_list));
      gtk_widget_show(menu_items);
      gtk_menu_append(GTK_MENU(send_to_pop), menu_items);
      send_to_list_next(in_use_send_to_list);
    }
  
    /* insert a line */
    menu_items = gtk_menu_item_new();
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);
	
    menu_items = gtk_menu_item_new_with_label("Properties...");
    gtk_signal_connect(GTK_OBJECT(menu_items), "activate",
      GTK_SIGNAL_FUNC(pop_prop_cb), (gpointer)last_FileDataList);
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);

  /* case when there is more than one file */
  } else {  
  
    menu_items = gtk_menu_item_new_with_label("Copy");
    gtk_signal_connect(GTK_OBJECT(menu_items), "activate",
      GTK_SIGNAL_FUNC(pop_copy_cb), (gpointer)last_FileDataList);
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);  
  
    menu_items = gtk_menu_item_new_with_label("Cut");
    gtk_signal_connect(GTK_OBJECT(menu_items), "activate",
      GTK_SIGNAL_FUNC(pop_cut_cb), (gpointer)last_FileDataList);
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);

    menu_items = gtk_menu_item_new_with_label("Delete");
    gtk_signal_connect(GTK_OBJECT(menu_items), "activate",
      GTK_SIGNAL_FUNC(pop_del_cb), (gpointer)last_FileDataList);
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);

    /* insert a line */
    menu_items = gtk_menu_item_new();
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);

    /* for the send to sub menu */    
    menu_items = gtk_menu_item_new_with_label("Send to");
    gtk_widget_show(menu_items);
    gtk_menu_append(GTK_MENU(pop_menu), menu_items);
    send_to_pop = gtk_menu_new();
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_items),send_to_pop);
    send_to_list_init(in_use_send_to_list);
    while(send_to_list_is_end(in_use_send_to_list) == FALSE)
    {
      menu_items = gtk_menu_item_new_with_label(send_to_command(send_to_list_current(in_use_send_to_list)));
      gtk_signal_connect(GTK_OBJECT(menu_items), "activate",
        GTK_SIGNAL_FUNC(pop_send_to_cb), (gpointer)send_to_list_current(in_use_send_to_list));
      gtk_widget_show(menu_items);
      gtk_menu_append(GTK_MENU(send_to_pop), menu_items);
      send_to_list_next(in_use_send_to_list);
    }
  }
  }
  
  gtk_menu_popup(GTK_MENU(pop_menu), NULL, NULL, NULL, NULL,
    button, activate_time);
}
