/****************************************/
/*  PreferenceSendTo.c body             */
/*					*/
/*  Description : The send to part of   */
/*		  the preference dialog	*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

/**********************************************************************/
/* TO DO                                                              */
/* - improve drag & drop (don't use focus_row because when nothing is */
/* selected as src but dst is correct focus_row is correct and a move */
/*   is done)                                                         */
/**********************************************************************/

guint preference_send_to_addedit_get_type ();
GtkWidget *preference_send_to_new(GtkWidget *parent_window,SendToList *to_use_send_to);

static void addedit_destroy_cb(PreferenceSendToAddedit
  *pPreferenceSendToAddedit, PreferenceSendTo *pPreferenceSendTo);
static void addedit_add_type_cb(PreferenceSendToAddedit
  *pPreferenceSendToAddedit,SendTo *beforetype, SendTo *newtype,
  PreferenceSendTo *pPreferenceSendTo);  
static void addedit_remove_type_cb(PreferenceSendToAddedit
  *pPreferenceSendToAddedit,SendTo *removetype,
  PreferenceSendTo *pPreferenceSendTo);
static void send_to_add_cb(GtkWidget *widget,PreferenceSendTo *pPreferenceSendTo);
static void send_to_edit_cb(GtkWidget *widget,PreferenceSendTo *pPreferenceSendTo);
static void send_to_remove_cb(GtkWidget *widget,PreferenceSendTo *pPreferenceSendTo);
static void send_to_move_up_cb(GtkWidget *widget,PreferenceSendTo *pPreferenceSendTo);
static void send_to_move_down_cb(GtkWidget *widget,PreferenceSendTo *pPreferenceSendTo);

static void preference_send_to_class_init(PreferenceSendToClass *pPreferenceSendToClass);
static void preference_send_to_init(PreferenceSendTo *pPreferenceSendTo);
static void destroy_cb(GtkWidget *GtkWidget,PreferenceSendTo *pPreferenceSendTo);

static void target_drag_data_received(GtkWidget *widget,GdkDragContext *context,
  gint x,gint y,GtkSelectionData *data,guint info,guint time,PreferenceSendTo *pPreferenceSendTo);
static gboolean target_drag_drop(GtkWidget *widget,GdkDragContext *context,
  gint x,gint y,guint time,PreferenceSendTo *pPreferenceSendTo);
static gboolean target_drag_motion(GtkWidget *widget,GdkDragContext *context,
  gint x,gint y,guint time,PreferenceSendTo *pPreferenceSendTo);
static void source_drag_data_get(GtkWidget *widget,GdkDragContext *context,
  GtkSelectionData *selection_data,guint info,guint time,gpointer data);

/*************************************************************/
guint preference_send_to_get_type()
{
  static guint preference_send_to_type = 0;

  if (!preference_send_to_type)
  {
    GtkTypeInfo preference_send_to_info =
    {
       "PreferenceSendTo",
       sizeof (PreferenceSendTo),
       sizeof (PreferenceSendToClass),
       (GtkClassInitFunc)  preference_send_to_class_init,
       (GtkObjectInitFunc) preference_send_to_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     preference_send_to_type = gtk_type_unique(gtk_hbox_get_type(), &preference_send_to_info);
  }
  return preference_send_to_type;
}
/*************************************************************/

/*************************************************************/
static void preference_send_to_class_init(PreferenceSendToClass *class)
{
  /* here we create the new signals if we want */    
}
/*************************************************************/
    
/***********************************************************/
/* Remove the selected command from the clist and from the */
/* send to list.                                           */
static void send_to_remove_cb(GtkWidget *widget,PreferenceSendTo *pPreferenceSendTo)
{ SendTo *tmp_type;
  gint vselect_row;

  /* if there is something selected */
  if(pPreferenceSendTo->list->selection != NULL)
    vselect_row = (gint)((GList *)(pPreferenceSendTo->list->selection)->data);
  else return;

  tmp_type = gtk_clist_get_row_data(pPreferenceSendTo->list,vselect_row);
  /* if the one we want to remove is in edition, exit */
  if((pPreferenceSendTo->addedit) &&
     (PREFERENCE_SEND_TO_ADDEDIT(pPreferenceSendTo->addedit)->lasttype == tmp_type)) return;

  gtk_clist_remove(pPreferenceSendTo->list,vselect_row);
  /* remove the command */
  send_to_list_remove_send_to(pPreferenceSendTo->to_use_send_to,tmp_type);
}
/***********************************************************/

/*********************************/
static void addedit_destroy_cb(PreferenceSendToAddedit
  *pPreferenceSendToAddedit, PreferenceSendTo *pPreferenceSendTo)
{
  gtk_widget_set_sensitive(pPreferenceSendTo->add,TRUE);
  gtk_widget_set_sensitive(pPreferenceSendTo->edit,TRUE);
  gtk_widget_set_sensitive(pPreferenceSendTo->remove,TRUE);
  pPreferenceSendTo->addedit = NULL;
}
/*********************************/

/*****************************************************/
static void addedit_add_type_cb(PreferenceSendToAddedit
  *pPreferenceSendToAddedit,SendTo *beforetype, SendTo *newtype,
  PreferenceSendTo *pPreferenceSendTo)
{ gint vselect_row;

  if(beforetype == NULL) {
    vselect_row = gtk_clist_append(pPreferenceSendTo->list,&newtype->command_str);
  } else { 
    vselect_row = gtk_clist_find_row_from_data(pPreferenceSendTo->list,
      beforetype);
    vselect_row = gtk_clist_insert(pPreferenceSendTo->list,
      vselect_row+1,&newtype->command_str);
  }

  /* attach the datas for making a relation between this list and the */
  /* existing send to commands */
  gtk_clist_set_row_data(pPreferenceSendTo->list,vselect_row,newtype);
}
/*****************************************************/

/******************************************************************/
static void addedit_remove_type_cb(PreferenceSendToAddedit
  *pPreferenceSendToAddedit,SendTo *removetype,
  PreferenceSendTo *pPreferenceSendTo)
{ gint vselect_row;

  g_print("addedit_remove_type_cb called\n");
  vselect_row = gtk_clist_find_row_from_data(pPreferenceSendTo->list,
    removetype);
  gtk_clist_remove(pPreferenceSendTo->list,vselect_row);
}
/******************************************************************/

/*****************************/
/* add a command to the list */
static void send_to_add_cb(GtkWidget *widget,PreferenceSendTo *pPreferenceSendTo)
{ SendTo *beforetype;
  gint vselect_row;

  /* only one dialog open at a time */
  if(pPreferenceSendTo->addedit) return;

  gtk_widget_set_sensitive(pPreferenceSendTo->add,FALSE);
  gtk_widget_set_sensitive(pPreferenceSendTo->edit,FALSE);

  /* if there is something selected, add after that type */
  if(pPreferenceSendTo->list->selection != NULL){
    vselect_row = (gint)((GList *)(pPreferenceSendTo->list->selection)->data);
    beforetype = gtk_clist_get_row_data(pPreferenceSendTo->list,vselect_row);
  /* else add at the end */
  } else beforetype = NULL;

  pPreferenceSendTo->addedit = preference_send_to_addedit_new
    (pPreferenceSendTo->to_use_send_to,beforetype,NULL);

  gtk_signal_connect (GTK_OBJECT(pPreferenceSendTo->addedit), "add_type",
                      GTK_SIGNAL_FUNC (addedit_add_type_cb),
                      pPreferenceSendTo);

  gtk_signal_connect (GTK_OBJECT(pPreferenceSendTo->addedit), "remove_type",
                      GTK_SIGNAL_FUNC (addedit_remove_type_cb),
                      pPreferenceSendTo);

  gtk_signal_connect (GTK_OBJECT(pPreferenceSendTo->addedit), "destroy",
                      GTK_SIGNAL_FUNC (addedit_destroy_cb),
                      pPreferenceSendTo);

  gtk_widget_show(pPreferenceSendTo->addedit);
}
/*****************************/

/***************************/
/* edit a type to the list */
void send_to_edit_cb(GtkWidget *widget,PreferenceSendTo *pPreferenceSendTo)
{ SendTo *lasttype;
  gint vselect_row;

  /* only one dialog open at a time */
  if(pPreferenceSendTo->addedit) return;

  /* if there is something selected, add after that type */
  if(pPreferenceSendTo->list->selection != NULL){
    vselect_row = (gint)((GList *)(pPreferenceSendTo->list->selection)->data);
    lasttype = gtk_clist_get_row_data(pPreferenceSendTo->list,vselect_row);
  /* else add at the end */
  } else return;

  gtk_widget_set_sensitive(pPreferenceSendTo->add,FALSE);
  gtk_widget_set_sensitive(pPreferenceSendTo->edit,FALSE);
  gtk_widget_set_sensitive(pPreferenceSendTo->remove,FALSE);

  pPreferenceSendTo->addedit = preference_send_to_addedit_new
    (pPreferenceSendTo->to_use_send_to,lasttype,lasttype);

  gtk_signal_connect (GTK_OBJECT(pPreferenceSendTo->addedit), "add_type",
                      GTK_SIGNAL_FUNC (addedit_add_type_cb),
                      pPreferenceSendTo);

  gtk_signal_connect (GTK_OBJECT(pPreferenceSendTo->addedit), "remove_type",
                      GTK_SIGNAL_FUNC (addedit_remove_type_cb),
                      pPreferenceSendTo);

  gtk_signal_connect (GTK_OBJECT(pPreferenceSendTo->addedit), "destroy",
                      GTK_SIGNAL_FUNC (addedit_destroy_cb),
                      pPreferenceSendTo);

  gtk_widget_show(pPreferenceSendTo->addedit);
}
/**************************/

/***************************/
/* move the select type up */
void send_to_move_up_cb(GtkWidget *widget,PreferenceSendTo *pPreferenceSendTo)
{ gint vselect_row;

  /* if there is something selected */
  if(pPreferenceSendTo->list->selection != NULL)
    vselect_row = (gint)((GList *)(pPreferenceSendTo->list->selection)->data);
  else return;  
  /*g_print("row to move up : %d.\n");*/
  if(vselect_row > 0)
  {
    gtk_clist_freeze (pPreferenceSendTo->list);
    gtk_clist_unselect_row (pPreferenceSendTo->list,vselect_row,0);
    gtk_clist_swap_rows(pPreferenceSendTo->list,vselect_row-1,vselect_row);
    gtk_clist_select_row (pPreferenceSendTo->list,vselect_row-1,0);
    gtk_clist_thaw   (pPreferenceSendTo->list);
    /* move up in the send to too */
    /* swap in the send_to list */
    send_to_list_swap_send_to(pPreferenceSendTo->to_use_send_to,
      gtk_clist_get_row_data(pPreferenceSendTo->list,vselect_row),
      gtk_clist_get_row_data(pPreferenceSendTo->list,vselect_row-1));
  }  
}
/***************************/

/*****************************/
/* move the select type down */
void send_to_move_down_cb(GtkWidget *widget,PreferenceSendTo *pPreferenceSendTo)
{ gint vselect_row;

  /* if there is something selected */
  if(pPreferenceSendTo->list->selection != NULL)
    vselect_row = (gint)((GList *)(pPreferenceSendTo->list->selection)->data);
  else return;  

  if(vselect_row < (pPreferenceSendTo->list->rows - 1) )
  {
    gtk_clist_freeze (pPreferenceSendTo->list);
    gtk_clist_unselect_row (pPreferenceSendTo->list,vselect_row,0);
    gtk_clist_swap_rows(pPreferenceSendTo->list,vselect_row,vselect_row+1);
    gtk_clist_select_row (pPreferenceSendTo->list,vselect_row+1,0);
    gtk_clist_thaw   (pPreferenceSendTo->list);
    /* move down in the send to too */
    /* swap in the send_to list */
    send_to_list_swap_send_to(pPreferenceSendTo->to_use_send_to,
      gtk_clist_get_row_data(pPreferenceSendTo->list,vselect_row),
      gtk_clist_get_row_data(pPreferenceSendTo->list,vselect_row+1));
  }  
}
/*****************************/

/************************************************/
/* Create a widget for the preference send to   */
/* Return : the widget			        */

static char * up_xpm[] = {
"14 11 5 1",
" 	c None",
".	c #C6C6C6",
"+	c #969696",
"@	c #000000",
"#	c #FFFFFF",
"      .+      ",
"     .@@+     ",
"    .@@@@+    ",
"   .@@@@@@+   ",
"  .@@@@@@@@+  ",
" .@@@@@@@@@@+ ",
".@@@@@@@@@@@@.",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
".#############"};

static char * down_xpm[] = {
"14 11 5 1",
" 	c None",
".	c #C6C6C6",
"+	c #969696",
"@	c #000000",
"#	c #FFFFFF",
".++++++++++++.",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
" .@@@@@@@@@@# ",
"  .@@@@@@@@#  ",
"   .@@@@@@#   ",
"    .@@@@#    ",
"     .@@#     ",
"      .#      "};


/* use as parameters given by preference_send_to_new to 
   preference_send_to_init */
static SendToList  *current_to_use_send_to;
static GtkWidget   *current_parent_window;

/***********************************/
/* call when the widget is destroy */
static void destroy_cb(GtkWidget *Widget,PreferenceSendTo *pPreferenceSendTo)
{
  /* if there is an addedit dialog open, close it */
  if(pPreferenceSendTo->addedit)
    gnome_dialog_close(GNOME_DIALOG(pPreferenceSendTo->addedit));
}
/***********************************/

/**************************************************/
/* called when a SendTo is selected in the list   */
static void select_row_cb(GtkCList *clist,gint row,gint column,GdkEvent *event,
  PreferenceSendTo *pPreferenceSendTo)
{
  if((pPreferenceSendTo->addedit != NULL) &&
     (PREFERENCE_SEND_TO_ADDEDIT(pPreferenceSendTo->addedit)->lasttype == 
       (SendTo *)gtk_clist_get_row_data(pPreferenceSendTo->list,row)))
    gtk_widget_set_sensitive(pPreferenceSendTo->remove,FALSE);
}
/**************************************************/

/****************************************************/
/* called when a SendTo is unselected in the list   */
static void unselect_row_cb(GtkCList *clist,gint row,gint column,
  GdkEvent *event,PreferenceSendTo *pPreferenceSendTo)
{
  if(GTK_WIDGET_IS_SENSITIVE(pPreferenceSendTo->remove) == FALSE)
    gtk_widget_set_sensitive(pPreferenceSendTo->remove,TRUE);
}
/****************************************************/

enum {
  TARGET_SEND_TO
};

static GtkTargetEntry target_table[] = {
  { "SendTo", 0, TARGET_SEND_TO }
};

static guint n_targets = sizeof(target_table) / sizeof(target_table[0]);


void preference_send_to_init(PreferenceSendTo *pPreferenceSendTo)
{
  GtkWidget *buttons;
  GtkWidget *move_up;
  GtkWidget *move_down;
  GtkWidget *scrolled_widget;
  gchar     *titles[1] = {"Commands"};
  SendTo    *tmptype = NULL;
  /* __ FOR THE UP & DOWN PIXMAP ____*/
  GdkPixmap *updown_pixmap;
  GdkBitmap *updown_mask;
  GtkWidget *updown_pixmap_widget;

  /* take in account parameters */
  pPreferenceSendTo->to_use_send_to = current_to_use_send_to;

  gtk_box_set_homogeneous(GTK_BOX(pPreferenceSendTo),FALSE);
  gtk_box_set_spacing(GTK_BOX(pPreferenceSendTo),0);

  gtk_signal_connect (GTK_OBJECT(pPreferenceSendTo), "destroy",
                      GTK_SIGNAL_FUNC(destroy_cb),
                      pPreferenceSendTo);

  buttons = gtk_vbox_new (FALSE, 0);

  pPreferenceSendTo->edit = gtk_button_new_with_label ("Edit");
  gtk_widget_show(pPreferenceSendTo->edit);
  gtk_box_pack_start (GTK_BOX (buttons), pPreferenceSendTo->edit, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (pPreferenceSendTo->edit), "clicked",
                      GTK_SIGNAL_FUNC (send_to_edit_cb),
                      pPreferenceSendTo);

  pPreferenceSendTo->add = gtk_button_new_with_label ("Add");
  gtk_widget_show(pPreferenceSendTo->add);
  gtk_box_pack_start (GTK_BOX (buttons), pPreferenceSendTo->add, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (pPreferenceSendTo->add), "clicked",
                      GTK_SIGNAL_FUNC (send_to_add_cb),
                      pPreferenceSendTo);


  pPreferenceSendTo->remove = gtk_button_new_with_label ("Remove");
  gtk_widget_show(pPreferenceSendTo->remove);
  gtk_box_pack_start (GTK_BOX (buttons), pPreferenceSendTo->remove, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (pPreferenceSendTo->remove), "clicked",
                      GTK_SIGNAL_FUNC (send_to_remove_cb),
                      pPreferenceSendTo);
  /*** create the up button ***/
  updown_pixmap = gdk_pixmap_create_from_xpm_d (
    current_parent_window->window,&updown_mask,NULL,up_xpm);

  /* create a widget with the pixmap */
  updown_pixmap_widget = gtk_pixmap_new(updown_pixmap,updown_mask);
  /* free the pixmap */
  gdk_pixmap_unref(updown_pixmap); gdk_bitmap_unref(updown_mask);

  move_up = gtk_button_new ();
  gtk_container_add(GTK_CONTAINER(move_up),updown_pixmap_widget);
  gtk_widget_show(updown_pixmap_widget);
  gtk_widget_show(move_up);
  gtk_box_pack_start (GTK_BOX (buttons), move_up, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (move_up), "clicked",
                      GTK_SIGNAL_FUNC (send_to_move_up_cb),
                      pPreferenceSendTo);
		      
  /*** create the down button ***/
  updown_pixmap = gdk_pixmap_create_from_xpm_d (
    current_parent_window->window,&updown_mask,NULL,down_xpm);

  /* create a widget with the pixmap */
  updown_pixmap_widget = gtk_pixmap_new(updown_pixmap,updown_mask);
  /* free the pixmap */
  gdk_pixmap_unref(updown_pixmap); gdk_bitmap_unref(updown_mask);

  move_down = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(move_down),updown_pixmap_widget);
  gtk_widget_show(updown_pixmap_widget);
  gtk_widget_show(move_down);
  gtk_box_pack_start (GTK_BOX (buttons), move_down, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (move_down), "clicked",
                      GTK_SIGNAL_FUNC (send_to_move_down_cb),
                      pPreferenceSendTo);

  pPreferenceSendTo->list = GTK_CLIST(gtk_clist_new_with_titles(1,titles));

  gtk_signal_connect (GTK_OBJECT (pPreferenceSendTo->list), "select_row",
                      GTK_SIGNAL_FUNC (select_row_cb),
                      pPreferenceSendTo);

  gtk_signal_connect (GTK_OBJECT (pPreferenceSendTo->list), "unselect_row",
                      GTK_SIGNAL_FUNC (unselect_row_cb),
                      pPreferenceSendTo);

  gtk_clist_set_row_height (pPreferenceSendTo->list, 18);

  gtk_widget_show (GTK_WIDGET(pPreferenceSendTo->list));

  /* we fill with all the existing commands */
  send_to_list_init(pPreferenceSendTo->to_use_send_to);
  while(send_to_list_is_end(pPreferenceSendTo->to_use_send_to) == FALSE)
  {
    /* add at the end */
    addedit_add_type_cb(NULL,NULL, send_to_list_current(
      pPreferenceSendTo->to_use_send_to),pPreferenceSendTo);

    send_to_list_next(pPreferenceSendTo->to_use_send_to);
  }

  /* define the scrolled policy */
  scrolled_widget = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_widget),
    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  gtk_widget_show(scrolled_widget);
  gtk_container_add(GTK_CONTAINER(scrolled_widget), GTK_WIDGET(
    pPreferenceSendTo->list));
    
  gtk_box_pack_start (GTK_BOX(pPreferenceSendTo),scrolled_widget,TRUE,TRUE,0);
  gtk_box_pack_start (GTK_BOX(pPreferenceSendTo), buttons, FALSE, TRUE, 0);

  gtk_container_border_width (GTK_CONTAINER(pPreferenceSendTo), 4);
  gtk_container_border_width (GTK_CONTAINER(buttons), 4);

  gtk_widget_show(buttons);
  /* no addedit dialog open at the startup */
  pPreferenceSendTo->addedit = NULL; 
  
  /* for the drag&drop */
  /* set the list as a destination */
  gtk_drag_dest_set(GTK_WIDGET(pPreferenceSendTo->list), 0, 
    target_table, n_targets, 0);

  gtk_signal_connect(GTK_OBJECT(pPreferenceSendTo->list),"drag_motion",
    GTK_SIGNAL_FUNC(target_drag_motion),pPreferenceSendTo);

  gtk_signal_connect(GTK_OBJECT(pPreferenceSendTo->list),"drag_data_received",
    GTK_SIGNAL_FUNC(target_drag_data_received),pPreferenceSendTo);

  gtk_signal_connect(GTK_OBJECT(pPreferenceSendTo->list),"drag_drop",
    GTK_SIGNAL_FUNC(target_drag_drop),pPreferenceSendTo);
  /* set the list as a source */
  gtk_drag_source_set(GTK_WIDGET(pPreferenceSendTo->list),GDK_BUTTON1_MASK|GDK_BUTTON3_MASK,
    target_table, n_targets, GDK_ACTION_COPY | GDK_ACTION_MOVE);

  gtk_signal_connect(GTK_OBJECT(pPreferenceSendTo->list),"drag_data_get",
    GTK_SIGNAL_FUNC(source_drag_data_get),pPreferenceSendTo);
}
/************************************************/

/*****************************************************/
/* Create a new widget for the preference send to    */
/* Return : the widget			             */
GtkWidget *preference_send_to_new(GtkWidget *parent_window,SendToList *to_use_send_to)
{
  current_parent_window = parent_window;
  current_to_use_send_to = to_use_send_to;

  return GTK_WIDGET (gtk_type_new( preference_send_to_get_type() ));
}
/*****************************************************/

/*******************************************/
/* UNDER DEVELOPPEMENT FOR THE DRAG & DROP */
/* not done                                */
/*******************************************/

gint start_row;

static gboolean target_drag_motion(GtkWidget *widget,GdkDragContext *context,
  gint x,gint y,guint time,PreferenceSendTo *pPreferenceSendTo)
{ 
  GtkWidget *source_widget;
  gint column;
  source_widget = gtk_drag_get_source_widget(context);
  
  /*g_print("motion called.\n");*/
  if((GTK_WIDGET(pPreferenceSendTo->list) == source_widget) &&
     (pPreferenceSendTo->list->rows))
  {
    gdk_drag_status (context, context->suggested_action, time);
    return TRUE;
  } else return FALSE;
}

static gboolean target_drag_drop(GtkWidget *widget,GdkDragContext *context,
  gint x,gint y,guint time,PreferenceSendTo *pPreferenceSendTo)
{
  /*g_print("drop called.\n");*/

  if((context->targets) && (pPreferenceSendTo->list->rows))
  {
    gtk_drag_get_data(widget,context,GPOINTER_TO_INT(context->targets->data),time);
    return TRUE;
  }
  return FALSE;
}

static void target_drag_data_received(GtkWidget *widget,GdkDragContext *context,
  gint x,gint y,GtkSelectionData *data,guint info,guint time,PreferenceSendTo *pPreferenceSendTo)
{
  GtkWidget *source_widget;
  gint row,column,focus_row;

  /*g_print("drag_received called at (%d,%d).\n",x,y);*/
  source_widget = gtk_drag_get_source_widget(context);

  if((info == TARGET_SEND_TO) && (source_widget) && 
     (GTK_WIDGET(pPreferenceSendTo->list) == source_widget) &&
     (pPreferenceSendTo->list->rows != 0))
  {
    gtk_clist_get_selection_info(pPreferenceSendTo->list,x,
      y-pPreferenceSendTo->list->column_title_area.height,&row,&column);

    if((row >= pPreferenceSendTo->list->rows) || (row < 0))
    {
      gtk_drag_finish (context, FALSE, FALSE, time);
      return;
    }

    focus_row = pPreferenceSendTo->list->focus_row;
    /*g_print("we take the DROP in account. line %d to %d\n",focus_row,row);*/

    send_to_list_move_send_to(pPreferenceSendTo->to_use_send_to,
      gtk_clist_get_row_data(pPreferenceSendTo->list,focus_row),
      gtk_clist_get_row_data(pPreferenceSendTo->list,row));

    gtk_clist_row_move(pPreferenceSendTo->list,focus_row,row);

    gtk_drag_finish (context, TRUE, FALSE, time);
  } else {
    /*g_print("the DROP go to TRASH\n");*/
    gtk_drag_finish (context, FALSE, FALSE, time);
  }
}


static void source_drag_data_get(GtkWidget *widget,GdkDragContext *context,
  GtkSelectionData *selection_data,guint info,guint time,gpointer data)
{
  /*g_print("get_data called.\n");*/
  if(info == TARGET_SEND_TO)
    gtk_selection_data_set(selection_data,selection_data->target,
      8, "SendTo swapping", 15);
}
