/****************************************/
/*  PreferenceSendToAddEdit.c body      */
/*					*/
/*  Description : Bouton Add and Edit of*/
/*	          the send to part of   */
/*		  the preference dialog	*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

guint preference_send_to_addedit_get_type ();

void preference_send_to_addedit_ok_cb(GtkWidget *widget,
  PreferenceSendToAddedit *pPreferenceSendToAddedit);

void preference_send_to_addedit_cancel_cb(GtkWidget *widget,
  PreferenceSendToAddedit *pPreferenceSendToAddedit);

GtkWidget *preference_send_to_addedit_new(SendToList *pSendToList,
  SendTo *beforetype, SendTo *lasttype);

static void preference_send_to_addedit_class_init(
  PreferenceSendToAddeditClass *pPreferenceSendToAddeditClass);

static void preference_send_to_addedit_init(
  PreferenceSendToAddedit *pPreferenceSendToAddedit);

static void destroy_cb(GtkWidget *Widget, 
  PreferenceSendToAddedit *pPreferenceSendToAddedit);

/*************************************************************/
guint preference_send_to_addedit_get_type ()
{
  static guint preference_send_to_addedit_type = 0;

  if (!preference_send_to_addedit_type)
  {
    GtkTypeInfo preference_send_to_addedit_info =
    {
       "PreferenceSendToAddedit",
       sizeof (PreferenceSendToAddedit),
       sizeof (PreferenceSendToAddeditClass),
       (GtkClassInitFunc)  preference_send_to_addedit_class_init,
       (GtkObjectInitFunc) preference_send_to_addedit_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     preference_send_to_addedit_type = gtk_type_unique(gnome_dialog_get_type(), &preference_send_to_addedit_info);
  }
  return preference_send_to_addedit_type;
}
/*************************************************************/

/*************************************************************/
enum { ADD_TYPE_SIGNAL, REMOVE_TYPE_SIGNAL, LAST_SIGNAL};
static gint preference_send_to_addedit_signals[LAST_SIGNAL] = { 0 };

static void one_send_to_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*one_send_to_marshal_func)(PreferenceSendToAddedit *,
      SendTo *, gpointer);
    one_send_to_marshal_func rfunc = (one_send_to_marshal_func) func;
    (*rfunc) (PREFERENCE_SEND_TO_ADDEDIT(object),
	      GTK_VALUE_POINTER(args[0]), func_data);
}

static void two_send_to_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*two_send_to_marshal_func)(PreferenceSendToAddedit *,
      SendTo *, SendTo *, gpointer);
    two_send_to_marshal_func rfunc = (two_send_to_marshal_func) func;
    (*rfunc) (PREFERENCE_SEND_TO_ADDEDIT(object),
	      GTK_VALUE_POINTER(args[0]),GTK_VALUE_POINTER(args[1]), func_data );
}

static void preference_send_to_addedit_class_init(PreferenceSendToAddeditClass *class)
{
  /* here we create the new signals if we want */
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) class;

  preference_send_to_addedit_signals[ADD_TYPE_SIGNAL] = gtk_signal_new
            ("add_type",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (PreferenceSendToAddeditClass,add_type),
            two_send_to_marshal, GTK_TYPE_NONE, 2,GTK_TYPE_POINTER,
	    GTK_TYPE_POINTER);    

  preference_send_to_addedit_signals[REMOVE_TYPE_SIGNAL] = gtk_signal_new
            ("remove_type",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (PreferenceSendToAddeditClass,remove_type),
            one_send_to_marshal, GTK_TYPE_NONE, 1,GTK_TYPE_POINTER);    
	    
   gtk_object_class_add_signals(object_class, 
     preference_send_to_addedit_signals, LAST_SIGNAL);
   class->add_type = NULL;  class->remove_type = NULL;	    
}
/*************************************************************/

/* use as parameter given by preference_send_to_addedit_new
  to preference_send_to_addedit_init */
static SendTo *current_preference_send_to_addedit_beforetype;
static SendTo *current_preference_send_to_addedit_lasttype;
static SendToList *current_preference_send_to_addedit_to_use_send_to_list;

/******************************************/
/* call when the browse button is pressed */
static void browse_cancel_cb(GtkWidget *Widget,PreferenceSendToAddedit *pPreferenceSendToAddedit)
{ gtk_widget_destroy(pPreferenceSendToAddedit->browse_dialog); }

static void browse_destroy_cb(GtkWidget *Widget,PreferenceSendToAddedit *pPreferenceSendToAddedit)
{ pPreferenceSendToAddedit->browse_dialog = NULL; }

static void browse_ok_cb(GtkWidget *Widget,PreferenceSendToAddedit *pPreferenceSendToAddedit)
{ gchar *string;
  
  string = gtk_file_selection_get_filename(GTK_FILE_SELECTION(pPreferenceSendToAddedit->browse_dialog));
  /* if it is not a directory, we take it */
  if(string[strlen(string)-1] != '/')
  {
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(pPreferenceSendToAddedit->program)->entry),string);
    gtk_widget_destroy(pPreferenceSendToAddedit->browse_dialog);
  } 
}

static void program_browse_cb(GtkButton *button,PreferenceSendToAddedit *pPreferenceSendToAddedit)
{
  if(pPreferenceSendToAddedit->browse_dialog == NULL)
  {
    pPreferenceSendToAddedit->browse_dialog = gtk_file_selection_new("Choose the program");
    gtk_widget_show(pPreferenceSendToAddedit->browse_dialog);

    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(pPreferenceSendToAddedit->browse_dialog)->ok_button),
      "clicked",GTK_SIGNAL_FUNC(browse_ok_cb),pPreferenceSendToAddedit);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(pPreferenceSendToAddedit->browse_dialog)->cancel_button),
      "clicked",GTK_SIGNAL_FUNC(browse_cancel_cb),pPreferenceSendToAddedit);
    gtk_signal_connect(GTK_OBJECT(pPreferenceSendToAddedit->browse_dialog),
      "destroy",GTK_SIGNAL_FUNC(browse_destroy_cb),pPreferenceSendToAddedit);
  }
}
/******************************************/

/****************************************/
/* Create a new dialog for add or edit. */
static void preference_send_to_addedit_init(
  PreferenceSendToAddedit *pPreferenceSendToAddedit)
{
  GtkWidget *label;
  GtkWidget *menu;
  GtkWidget *menu_item;
  gint i;
  GtkWidget  *table;
  GList      *strings = NULL;
  GList      *tmp_node;
  SendTo     *tmptype;
  gchar      *string;
  GtkWidget  *h1;

  /* use parameters */
  pPreferenceSendToAddedit->beforetype =
    current_preference_send_to_addedit_beforetype;  
  pPreferenceSendToAddedit->lasttype = 
    current_preference_send_to_addedit_lasttype;
  pPreferenceSendToAddedit->to_use_send_to = 
    current_preference_send_to_addedit_to_use_send_to_list;

  /* set the right title */
  if(pPreferenceSendToAddedit->lasttype == NULL)
    gtk_window_set_title(GTK_WINDOW(pPreferenceSendToAddedit),"New command");
  else
    gtk_window_set_title(GTK_WINDOW(pPreferenceSendToAddedit),"Edit command");

  /* add the OK and CANCEL buttons */
  gnome_dialog_append_buttons(GNOME_DIALOG(pPreferenceSendToAddedit),
    GNOME_STOCK_BUTTON_OK,GNOME_STOCK_BUTTON_CANCEL,NULL);

  /* Cancel button */
  gnome_dialog_button_connect(GNOME_DIALOG(pPreferenceSendToAddedit),1,
    GTK_SIGNAL_FUNC(preference_send_to_addedit_cancel_cb),pPreferenceSendToAddedit);
  /* Ok button */
  gnome_dialog_button_connect(GNOME_DIALOG(pPreferenceSendToAddedit),0,
    GTK_SIGNAL_FUNC(preference_send_to_addedit_ok_cb),pPreferenceSendToAddedit);

  /* connect the destroy signal of the dialog to destroy_cb */
  gtk_signal_connect(GTK_OBJECT(pPreferenceSendToAddedit), "destroy",
    GTK_SIGNAL_FUNC(destroy_cb),pPreferenceSendToAddedit);

  /* all the rest in a table */
  table = gtk_table_new(1,2,FALSE);
  gtk_widget_show(table);
  gtk_table_set_row_spacings(GTK_TABLE(table),8);
  gtk_table_set_col_spacings(GTK_TABLE(table),8);
  gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(pPreferenceSendToAddedit)->vbox),
    table, FALSE, TRUE, 0);

  /* Program */
  pPreferenceSendToAddedit->program_label=gtk_label_new("command");
  gtk_widget_show(pPreferenceSendToAddedit->program_label);
  gtk_table_attach_defaults(GTK_TABLE(table),
    pPreferenceSendToAddedit->program_label,0,1,0,1);
  h1 = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(h1);
  gtk_table_attach_defaults(GTK_TABLE(table),h1,1,2,0,1);
  pPreferenceSendToAddedit->program = gtk_combo_new();
  gtk_widget_show(pPreferenceSendToAddedit->program);
  /* fill the program combo list with all the all ready used program */  
  send_to_list_init(pPreferenceSendToAddedit->to_use_send_to);
  while(send_to_list_is_end(pPreferenceSendToAddedit->to_use_send_to) == FALSE)
  {
    if(send_to_list_current(pPreferenceSendToAddedit->to_use_send_to)->command_str)
    { 
      /* try to find this program in the all ready known list */
      for(tmp_node = strings; tmp_node && 
        strcmp(tmp_node->data,send_to_list_current(
	pPreferenceSendToAddedit->to_use_send_to)->command_str);
        tmp_node = g_list_next(tmp_node));
      /* if new add it */
      if(tmp_node == NULL)
        strings = g_list_append(strings,
          send_to_list_current(pPreferenceSendToAddedit->to_use_send_to)->command_str);
    }
    send_to_list_next(pPreferenceSendToAddedit->to_use_send_to);
  }
  if(strings != NULL)
  {
    gtk_combo_set_popdown_strings(GTK_COMBO(pPreferenceSendToAddedit->program),
      strings);
    g_list_free(strings);
  }
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(pPreferenceSendToAddedit->program)->entry),"");
  gtk_box_pack_start(GTK_BOX(h1),pPreferenceSendToAddedit->program,TRUE,TRUE,0);
  pPreferenceSendToAddedit->browse = gtk_button_new_with_label("browse...");
  gtk_widget_show(pPreferenceSendToAddedit->browse);
  pPreferenceSendToAddedit->browse_dialog = NULL;
  gtk_signal_connect(GTK_OBJECT(pPreferenceSendToAddedit->browse),"clicked",
    GTK_SIGNAL_FUNC(program_browse_cb),pPreferenceSendToAddedit);    
  gtk_box_pack_start(GTK_BOX(h1),pPreferenceSendToAddedit->browse,FALSE,FALSE,4);

  /* fill with the old values if we are in edit mode */
  if(pPreferenceSendToAddedit->lasttype)
  {
    if(pPreferenceSendToAddedit->lasttype->command_str)
      gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(pPreferenceSendToAddedit->program)->entry),pPreferenceSendToAddedit->lasttype->command_str);
  }

}
/****************************************/

/***************************************************************/
/* Create a dialog for add or edit. If lasttype = NULL then it */
/* is in add mode else, in edit mode.                          */
/* If beforetype = NULL, insert at the end.                    */
GtkWidget *preference_send_to_addedit_new(SendToList *pSendToList,
  SendTo *beforetype, SendTo *lasttype)
{
  current_preference_send_to_addedit_beforetype = beforetype;
  current_preference_send_to_addedit_lasttype = lasttype;
  current_preference_send_to_addedit_to_use_send_to_list = pSendToList;

  return GTK_WIDGET(gtk_type_new( preference_send_to_addedit_get_type() ));
}
/***************************************************************/

/*************************************/
/* Button ok pressed in the Add mode */
void preference_send_to_addedit_ok_cb(GtkWidget *widget,
  PreferenceSendToAddedit *pPreferenceSendToAddedit)
{
  SendTo *new_type,*vSendTo;
  gchar  *program_str;
  gint   pos;
  GtkWidget *error_dialog;
  
  /* program string */
  program_str = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(pPreferenceSendToAddedit->program)->entry));
  for(pos = 0; program_str[pos] && ((program_str[pos]==' ')||(program_str[pos]=='\t'));pos++);
  if(program_str[pos] == 0) program_str = NULL;

  /* we create the new type */
  new_type = send_to_new(program_str);
  
  if(new_type != NULL)
  {
    send_to_list_insert_send_to(pPreferenceSendToAddedit->to_use_send_to,
      pPreferenceSendToAddedit->beforetype, new_type);
	
    /* alerte the parent that we have inserted a new type */
    gtk_signal_emit(GTK_OBJECT(pPreferenceSendToAddedit),
      preference_send_to_addedit_signals[ADD_TYPE_SIGNAL],
	pPreferenceSendToAddedit->beforetype,
	new_type);

    /* if we are in edit mode and the new creation has worked */
    if(pPreferenceSendToAddedit->lasttype) { /* we are in edit mode */
      /* alerte the parent that we are going to suppress a type */
      gtk_signal_emit(GTK_OBJECT(pPreferenceSendToAddedit),
        preference_send_to_addedit_signals[REMOVE_TYPE_SIGNAL],
	  pPreferenceSendToAddedit->lasttype);
      /* remove the type */
      send_to_list_remove_send_to(pPreferenceSendToAddedit->to_use_send_to,
        pPreferenceSendToAddedit->lasttype);
    }
    gnome_dialog_close(GNOME_DIALOG(pPreferenceSendToAddedit));

  } else { /* we must alerte the user that a problem occured */ 

    error_dialog = gnome_message_box_new("Unknown ERROR",GNOME_MESSAGE_BOX_ERROR,
      GNOME_STOCK_BUTTON_OK,NULL);
    gnome_dialog_set_close(GNOME_DIALOG(error_dialog),TRUE);
    gnome_dialog_run(GNOME_DIALOG(error_dialog));
  }
}
/*************************************/

/*************************************************/
/* Button cancel pressed in the Add or Edit mode */
void preference_send_to_addedit_cancel_cb(GtkWidget *widget,
  PreferenceSendToAddedit *pPreferenceSendToAddedit)
{ gnome_dialog_close (GNOME_DIALOG(pPreferenceSendToAddedit)); }
/*************************************************/

/**************************************/
/* call when the dialog box is closed */
static void destroy_cb(GtkWidget *Widget, PreferenceSendToAddedit *pPreferenceSendToAddedit)
{ GSList *tmp_node;

  /* if the browse program dialog is still open, close it */
  if(pPreferenceSendToAddedit->browse_dialog)
    gtk_widget_destroy(pPreferenceSendToAddedit->browse_dialog);
}
/**************************************/
