/****************************************/
/*  SendTo.c body                       */
/*					*/
/*  Description : SendTo take care of   */
/*		  send to commands      */
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

/***************************************************************/
/* Create a new send to command.                               */
/* Return : NULL if impossible and gfile_errno can be set to : */
/* FILE_TYPE_PARAM_ERROR                                       */
SendTo *send_to_new(gchar *command_str)
{ SendTo *vSendTo;

  if(command_str == NULL)
  {
    gfile_errno = SEND_TO_PARAM_ERROR;
    return(NULL);
  }
 
  /* alloc mem for the new send to */
  vSendTo = g_malloc(sizeof(SendTo));
  
  vSendTo->command_str = g_strdup(command_str);
    
  return(vSendTo);
}
/***************************************************************/

/*********************/
/* Free a send to.   */
void send_to_free(SendTo *pSendTo)
{
  if(pSendTo == NULL) return;

  if(pSendTo->command_str != NULL)   g_free(pSendTo->command_str);

  /* free the send to structure */
  g_free(pSendTo);
}
/*********************/

/*********************************************************/
/* Return a copy of pSendTo                              */
/* or NULL if impossible and gfile_errno can be set to : */
/* SEND_TO_PARAM_ERROR                                   */
SendTo *send_to_copy(SendTo *pSendTo)
{
  if(pSendTo == NULL)
  {
    gfile_errno = SEND_TO_PARAM_ERROR;
    return(NULL);
  }
  return(send_to_new(pSendTo->command_str));
}
/*********************************************************/
