/**********************************************/
/*  SendToList.c body                         */
/*					      */
/*  Description : SendToList take care of     */
/*		  send to commands list       */
/*					      */
/*  Written by : Daniel Lacroix	              */
/*  GFile is under the GPL2 licence           */
/**********************************************/

#include "GFile.h"

/*******************************************/
/* Create a new send to system.            */
/* Return : the new allocated SendToList */
SendToList *send_to_list_new(void)
{ SendToList *vSendToList;

  vSendToList = g_malloc(sizeof(SendToList));
  vSendToList->list =  g_slist_alloc();
  return(vSendToList);
}
/*******************************************/

/****************************/
/* Free a send to system.   */
void send_to_list_free(SendToList *pSendToList)
{
  send_to_list_init(pSendToList);
  while(send_to_list_is_end(pSendToList) == FALSE)
  {
    send_to_free(send_to_list_current(pSendToList));
    send_to_list_next(pSendToList);
  }
  g_slist_free(pSendToList->list);
  g_free(pSendToList);
}
/****************************/

/***************************************/
/* insert after "before" in the list   */
/* if before = NULL, insert at the end */
void send_to_list_insert_send_to(SendToList *pSendToList,SendTo *before,SendTo *pSendTo)
{ GSList *last_list;
  GSList *new_list;

  new_list = g_malloc(sizeof(GSList));
  new_list->data = (gpointer)pSendTo;

  last_list = pSendToList->list;

  send_to_list_init(pSendToList);
  while((send_to_list_is_end(pSendToList) == FALSE) && 
        (send_to_list_current(pSendToList) != before))
  { 
    last_list = pSendToList->current;
    send_to_list_next(pSendToList);
  }

  if((before != NULL) && (send_to_list_current(pSendToList) == before))
  { 
    last_list = pSendToList->current;
    send_to_list_next(pSendToList);
  }

  new_list->next = pSendToList->current;
  last_list->next = new_list;
}
/***************************************/

/************************************************/
/* Remove a send to from the send to list       */
void send_to_list_remove_send_to(SendToList *pSendToList,
  SendTo *remove_type)
{   
  GSList *last_type;
  
  last_type = pSendToList->list;
  
  if((pSendToList == NULL) || (remove_type == NULL)) return;
  
  /* free the element in the list*/
  send_to_list_init(pSendToList);
  while((send_to_list_is_end(pSendToList) == FALSE) &&
        (send_to_list_current(pSendToList) != remove_type))
  {
    last_type = pSendToList->current;
    send_to_list_next(pSendToList);
  }
  if(send_to_list_is_end(pSendToList) == TRUE)
    return; /* type not present */
  else {  
   last_type->next = pSendToList->current->next;
    g_free(pSendToList->current);
  }    
  
  send_to_free(remove_type);
}    
/************************************************/

/***************************************************************/
/* swap two SendToList.                                        */
/* You CAN'T CALLED THIS IF YOU ARE USING send_to_list_next... */
void send_to_list_swap_send_to_list(SendToList *pSendToList1,
  SendToList *pSendToList2)
{ GSList *tmp;
 
  tmp = pSendToList1->list;
  pSendToList1->list = pSendToList2->list;
  pSendToList2->list = tmp;
}
/***************************************************************/

/**************************************/
/* swap two send to in a send to list */
void send_to_list_swap_send_to(SendToList *pSendToList,SendTo *type1,SendTo *type2)
{ GSList *data1_list,*data2_list;

  /* find type1 in the list */  
  send_to_list_init(pSendToList);
  while((send_to_list_is_end(pSendToList) == FALSE) && 
        (send_to_list_current(pSendToList) != type1))
  { send_to_list_next(pSendToList); }
  /* type not found */
  if(send_to_list_is_end(pSendToList) == TRUE) return;
  data1_list = pSendToList->current; 
  /* find type2 in the list */    
  send_to_list_init(pSendToList);
  while((send_to_list_is_end(pSendToList) == FALSE) && 
        (send_to_list_current(pSendToList) != type2))
  { send_to_list_next(pSendToList); }
  /* type not found */
  if(send_to_list_is_end(pSendToList) == TRUE) return;
  data2_list = pSendToList->current;
  /* data both exist => swap */
  data2_list->data = (gpointer)type1; data1_list->data = (gpointer)type2;
}
/**************************************/

/*************************************************************/
/* add all the send to commands of psrc at the end of pdest. */
/* If pdest is empty, this make a copy of psrc.              */
void send_to_list_append_send_to_list(SendToList *pdest,SendToList *psrc)
{ GSList *list_node;

  /* security check */
  if((pdest == NULL) || (psrc == NULL)) return;

  send_to_list_init(psrc);
  while(send_to_list_is_end(psrc) == FALSE)
  {
    send_to_list_append_send_to(pdest,send_to_copy(send_to_list_current(psrc)));
    send_to_list_next(psrc);
  }
}
/*************************************************************/

/****************************************/
/* Return a copy of psrc send to list   */
SendToList *send_to_list_copy(SendToList *psrc)
{ SendToList *vdest;

  vdest = send_to_list_new();
  send_to_list_append_send_to_list(vdest,psrc);
  return(vdest);
}
/****************************************/

/****************************************/
/* move SendTo at pos src to pos dst. */
void send_to_list_move_send_to(SendToList *pSendToList,SendTo *src,
  SendTo *dst)
{
  gint pos_src = 0;
  gint pos_dst = 0;
  GSList *src_node = NULL;
  GSList *dst_node = NULL;
  GSList *src_node_past = NULL;
  GSList *dst_node_past = NULL;
  GSList *list_node_past = NULL;

  /* security check */
  if((pSendToList == NULL) || (src == NULL) || (dst == NULL) ||
     (src == dst)) return;

  send_to_list_init(pSendToList);
  list_node_past = pSendToList->list;
  while((send_to_list_is_end(pSendToList) == FALSE) && 
        !(src_node && dst_node) )
  {    
    if(send_to_list_current(pSendToList) == src) {
      src_node = pSendToList->current;
      src_node_past = list_node_past;
    } else if(send_to_list_current(pSendToList) == dst) {
      dst_node = pSendToList->current;
      dst_node_past = list_node_past;
    }

    if(src_node == NULL) pos_src++;
    if(dst_node == NULL) pos_dst++;
   
    list_node_past = pSendToList->current;
    send_to_list_next(pSendToList);
  }

  /* if one or both of types not found, exit */
  if((src_node == NULL) || (dst_node == NULL)) return;

  /* remove the src */
  src_node_past->next = src_node->next;

  /* insert src at the right pos */
  if(pos_src > pos_dst)
  {
    src_node->next = dst_node;
    dst_node_past->next = src_node;
  } else {
    src_node->next = dst_node->next;
    dst_node->next = src_node;
  }
}
/*******************************************************/

/************************************************/
/* Save the send to list in the sendto file     */
/* filename is the full path name of the file   */
/* Return : FALSE = impossible; TRUE = job done */
/* if FALSE, gfile_errno can be set to          */
/*   SEND_TO_LIST_CREAT_FILE_ERROR              */
/*   SEND_TO_LIST_WRITE_FILE_ERROR              */
gboolean send_to_list_save_to_file(SendToList *pSendToList,gchar *filename)
{
  gint file_handle;
  gchar *string;

  file_handle = creat(filename,S_IREAD|S_IWRITE);
  if(file_handle == -1)
  { gfile_errno = SEND_TO_LIST_CREAT_FILE_ERROR; return(FALSE); }
  
  send_to_list_init(pSendToList);

  while(send_to_list_is_end(pSendToList) == FALSE)
  {
    /* write the current command in the file */
    string = g_strdup_printf("%s\n",send_to_command(send_to_list_current(pSendToList)));
    if(write(file_handle,string,strlen(string)) == -1)
    { close(file_handle); gfile_errno = SEND_TO_LIST_WRITE_FILE_ERROR; return(FALSE); }
    g_free(string);
    /* next send to commands */
    send_to_list_next(pSendToList);
  }

  close(file_handle);
  return(TRUE);
}
/************************************************/

/******************************************************/
/* Load the send to list from a file.                 */
/* filename is the full path name of the file         */
/* Return : NULL = impossible gfile_errno can be set  */
/*          to SEND_TO_LIST_FILE_TOO_BIG              */
/*             SEND_TO_LIST_FILE_OPEN_ERROR           */
/*          else job done and the send to list is fill*/
SendToList *send_to_list_load_from_file(gchar *filename)
{
  SendToList *vSendToList;
  gint       file_handle;
  SendTo     *vSendTo;
  gchar      *buf;
  gint       buf_size;
  guint      buf_pos;
  guint      line_nb = 1;
  guint      line_begin = 0;
  guint      info_begin = 0;
  guint      info_end = 0;
  gboolean   info_in_fill = FALSE;
  gboolean   info_nothing_fill = TRUE;
  guint8     info_num = 0;
  gchar      tmp_char;

  gchar    *command_str = NULL;
  gchar    *icon_name   = NULL; 
  
  vSendToList = send_to_list_new();

  /* if filename is NULL, then just return an empty list */
  if(filename == NULL) return(vSendToList);

  /* try to open the file */
  file_handle = open(filename,O_RDONLY);
  if(file_handle == -1)
  {
    send_to_list_free(vSendToList);
    gfile_errno = SEND_TO_LIST_FILE_OPEN_ERROR;
    return(NULL);
  }
  /* alloc a bloc of memory of the file size */
  buf_size = lseek(file_handle, 0, SEEK_END) + 1;
  if(buf_size == 0)
  {
    send_to_list_free(vSendToList);
    close(file_handle);
    gfile_errno = SEND_TO_LIST_FILE_OPEN_ERROR;
    return(NULL);
  }
  /* set the file handle at the begin */
  if(lseek(file_handle, 0, SEEK_SET) == -1)
  {
    send_to_list_free(vSendToList);
    close(file_handle);
    gfile_errno = SEND_TO_LIST_FILE_OPEN_ERROR;
    return(NULL);
  }
  if(buf_size-1 > SEND_TO_LIST_MAX_FILE_SIZE)
  {
    g_print("SendToList : file \"%s\" is too big, it might be a mistake.\n");
    send_to_list_free(vSendToList);
    close(file_handle);
    gfile_errno = SEND_TO_LIST_FILE_TOO_BIG;
    return(NULL);
  }
  buf = g_malloc(buf_size);
  buf[buf_size-1] = 0;
  if(read(file_handle,buf,buf_size-1) != buf_size-1)
  {
    g_free(buf);
    send_to_list_free(vSendToList);
    close(file_handle);
    gfile_errno = SEND_TO_LIST_FILE_OPEN_ERROR;
    return(NULL);
  }
  close(file_handle);

  /* replace all line feed by 0 */
  for(buf_pos = 0; buf_pos < buf_size; buf_pos++)
    if(buf[buf_pos]=='\n') buf[buf_pos] = 0;

  buf_pos = 0;
  while( buf_pos < buf_size )
  {
    switch(buf[buf_pos])
    {
      case '#' : 
        while(buf[++buf_pos] != 0);
        break;
      case 0   : 
        if(info_nothing_fill == FALSE)
	{
	  if(info_end <= info_begin) info_end = buf_pos;
          tmp_char = buf[info_end]; buf[info_end] = 0;
	  /*g_print("command : \"%s\"\n",&buf[info_begin]);*/
	  vSendTo = send_to_new(&buf[info_begin]);
	  send_to_list_append_send_to(vSendToList,vSendTo);
	  buf[info_end] = tmp_char;
	}
        line_nb++; buf_pos++;
        line_begin = buf_pos; info_end = buf_pos;
	info_nothing_fill = TRUE; info_in_fill = FALSE;
        break;
      case '\t' :
      case ' '  :
        if(info_in_fill){info_end = buf_pos; info_in_fill = FALSE;}
        buf_pos++;
        break;
      default :
        if((info_in_fill == FALSE) && (info_nothing_fill == FALSE))
        { info_in_fill = TRUE; 
	} else if(info_in_fill == FALSE)
	{info_begin = buf_pos; info_in_fill = TRUE; info_nothing_fill = FALSE;}
        buf_pos++;
        break;
    }
  }

  g_free(buf);
  return(vSendToList);
}
/******************************************************/
