/**********************************************/
/*  SendToList.c headers                      */
/*					      */
/*  Description : SendToList take care of     */
/*		  send to commands list       */
/*					      */
/*  Written by : Daniel Lacroix	              */
/*  GFile is under the GPL2 licence           */
/**********************************************/

#ifndef __SEND_TO_LIST_H__
#define __SEND_TO_LIST_H__

/* files bigger than that will be refused beeing 
   loaded by file_type_list_load_from_file */
#define SEND_TO_LIST_MAX_FILE_SIZE  500000

/* error number possible in gfile_errno */
#define SEND_TO_LIST_FILE_TOO_BIG       1
#define SEND_TO_LIST_FILE_OPEN_ERROR    2

#define SEND_TO_LIST_CREAT_FILE_ERROR   3
#define SEND_TO_LIST_WRITE_FILE_ERROR   4

typedef struct _SendToList SendToList;

struct _SendToList
{
  /* the list of all the send to commands */
  GSList *list;
  /* the two above are for going over the list */
  /* current type */
  GSList *current;
};

/* Create a new send to system.          */
/* Return : the new allocated SendToList */
SendToList *send_to_list_new(void);

/* Free a send to system. */
void send_to_list_free(SendToList *pSendToList);

/***************************************************************/
/************ FOR GOING OVER ALL THE SEND TO COMMANDS **********/

/* After a call to this procedure, send_to_list_current */
/* return the first SendTo                              */
#define send_to_list_init(pSendToList)    (pSendToList->current = g_slist_next(pSendToList->list))

/* Return the SendTo we are on */
#define send_to_list_current(pSendToList) ((SendTo *)pSendToList->current->data)

/* Go to the next SendTo */
#define send_to_list_next(pSendToList)    (pSendToList->current = g_slist_next(pSendToList->current))

/* say if there is no more SendTo */
#define send_to_list_is_end(pSendToList)  (pSendToList->current == NULL)

/***************************************************************/

/* insert after "before" in the list   */
/* if before = NULL, insert at the end */
void send_to_list_insert_send_to(SendToList *pSendToList,
  SendTo *before, SendTo *pSendTo);

/* add at the end of the list */
#define send_to_list_append_send_to(pSendToList,pSendTo) send_to_list_insert_send_to(pSendToList,NULL,pSendTo)

/* Remove a send to to the send to list     */
void send_to_list_remove_send_to(SendToList *pSendToList,
  SendTo *remove_type);

/* swap two SendToList.                                        */
/* You CAN'T CALLED THIS IF YOU ARE USING send_to_list_next... */
void send_to_list_swap_send_to_list(SendToList *pSendToList1,
  SendToList *pSendToList2);

/* swap two send to in a send to list */
void send_to_list_swap_send_to(SendToList *pSendToList,SendTo *type1,
  SendTo *type2);

/* add all the send to commands of psrc at the end of pdest. */
/* If pdest is empty, this make a copy of psrc.              */
void send_to_list_append_send_to_list(SendToList *pdest,
  SendToList *psrc);

/* Return a copy of psrc send to list */
SendToList *send_to_list_copy(SendToList *psrc);

/* move SendTo at pos src to pos dst. */
void send_to_list_move_send_to(SendToList *pSendToList,SendTo *src,
  SendTo *dst);

/* Save the send to list in the sendto file     */
/* filename is the full path name of the file   */
/* Return : FALSE = impossible; TRUE = job done */
/* if FALSE, gfile_errno can be set to          */
/*   SEND_TO_LIST_CREAT_FILE_ERROR              */
/*   SEND_TO_LIST_WRITE_FILE_ERROR              */
gboolean send_to_list_save_to_file(SendToList *pSendToList,gchar *filename);

/* Load the send to list from a file.                 */
/* filename is the full path name of the file         */
/* Return : NULL = impossible gfile_errno can be set  */
/*          to SEND_TO_LIST_FILE_TOO_BIG              */
/*             SEND_TO_LIST_FILE_OPEN_ERROR           */
/*          else job done and the send to list is fill*/
SendToList *send_to_list_load_from_file(gchar *filename);

#endif /* __SEND_TO_LIST_H__ */
