/****************************************/
/*  TestFileTypeList.c body             */
/*					*/
/*  Description : TestFileTypeList is a */
/*                program to test the   */
/*                  file type system    */
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

int main(int argc, char **argv)
{
  FileTypeList *FileTypeList1;
  FileTypeList *FileTypeList2;
  FileType     *tmp_file_type;

  file_type_set_icon_path("/root/programmation/GFile/icons/");

  gnome_init("TestFileTypeList", "test", argc,argv);

  g_print("Create FileTypeList1\n");
  FileTypeList1 = file_type_list_new();

  g_print("Test if FileTypeList1 is empty\n");
  file_type_list_init(FileTypeList1);
  if(file_type_list_is_end(FileTypeList1)) 
    g_print("> FileTypeList1 is empty\n");
  else
    g_print("> FileTypeList1 is NOT empty = BUG\n");

  g_print("Add two file type to FileTypeList1\n");
  tmp_file_type = file_type_new(USER_DEFINED,"STRING1",NULL,
    "/root/programmation/GFile/icons/unknow.xpm",FALSE,NULL,NULL);
  if(tmp_file_type == NULL) 
  { g_print("ERROR loading /root/programmation/GFile/icons/unknow.xpm.\n");
    exit(1);
  }
  file_type_list_append_file_type(FileTypeList1,tmp_file_type);
  tmp_file_type = file_type_new(USER_DEFINED,"STRING2",NULL,
    "exclam.xpm",FALSE,NULL,NULL);
  if(tmp_file_type == NULL)
  { g_print("ERROR loading exclam.xpm, relative icon.\n");
    exit(1);
  }
  file_type_list_append_file_type(FileTypeList1,tmp_file_type);
  file_type_list_init(FileTypeList1);
  while(file_type_list_is_end(FileTypeList1) == FALSE)
  {
    g_print("> %s\n",file_type_list_current(FileTypeList1)->type_str);
    file_type_list_next(FileTypeList1);
  }

  g_print("Insert one type after the first of FileTypeList1\n");
  file_type_list_init(FileTypeList1);

  tmp_file_type = file_type_new(USER_DEFINED,"STRING1.5",NULL,
    "unknow.xpm",FALSE,NULL,NULL); 
  file_type_list_init(FileTypeList1);
  file_type_list_insert_file_type(FileTypeList1,file_type_list_current(FileTypeList1),tmp_file_type);
  file_type_list_init(FileTypeList1);
  while(file_type_list_is_end(FileTypeList1) == FALSE)
  {
    g_print("> %s\n",file_type_list_current(FileTypeList1)->type_str);
    file_type_list_next(FileTypeList1);
  }		  

  g_print("Swap file type 1 and file type 3 in FileTypeList1\n");
  file_type_list_init(FileTypeList1);
  tmp_file_type = file_type_list_current(FileTypeList1);
  file_type_list_next(FileTypeList1); file_type_list_next(FileTypeList1);
  file_type_list_swap_file_type(FileTypeList1,file_type_list_current(FileTypeList1),tmp_file_type);
  file_type_list_init(FileTypeList1);
  while(file_type_list_is_end(FileTypeList1) == FALSE)
  {
    g_print("> %s\n",file_type_list_current(FileTypeList1)->type_str);
    file_type_list_next(FileTypeList1);
  }		  

  g_print("Create FileTypeList2\n");
  FileTypeList2 = file_type_list_new();

  g_print("Add two type to FileTypeList2\n");
  tmp_file_type = file_type_new(USER_DEFINED,"STRING3",NULL,
    "unknow.xpm",FALSE,NULL,NULL);
  file_type_list_append_file_type(FileTypeList2,tmp_file_type);
  tmp_file_type = file_type_new(USER_DEFINED,"STRING4",NULL,
    "exclam.xpm",FALSE,NULL,NULL);
  file_type_list_append_file_type(FileTypeList2,tmp_file_type);
  file_type_list_init(FileTypeList2);
  while(file_type_list_is_end(FileTypeList2) == FALSE)
  {
    g_print("> %s\n",file_type_list_current(FileTypeList2)->type_str);    
    file_type_list_next(FileTypeList2);
  }		  

  g_print("Swap FileTypeList1 & FileTypeList2\n");
  file_type_list_swap_file_type_list(FileTypeList1,FileTypeList2);
  
  g_print("New content of FileTypeList1\n");
  file_type_list_init(FileTypeList1);
  while(file_type_list_is_end(FileTypeList1) == FALSE)
  {
    g_print("> %s\n",file_type_list_current(FileTypeList1)->type_str);    
    file_type_list_next(FileTypeList1);
  }		  
  
  g_print("New content of FileTypeList2\n");
  file_type_list_init(FileTypeList2);
  while(file_type_list_is_end(FileTypeList2) == FALSE)
  {
    g_print("> %s\n",file_type_list_current(FileTypeList2)->type_str);
    file_type_list_next(FileTypeList2);
  }		  

  g_print("Free first file type of FileTypeList1\n");
  file_type_list_init(FileTypeList1);
  file_type_list_remove_file_type(FileTypeList1,file_type_list_current(FileTypeList1));
  file_type_list_init(FileTypeList1);
  while(file_type_list_is_end(FileTypeList1) == FALSE)
  {
    g_print("> %s\n",file_type_list_current(FileTypeList1)->type_str);
    file_type_list_next(FileTypeList1);
  }		  
  g_print("Free first file type of FileTypeList1\n");
  file_type_list_init(FileTypeList1);
  file_type_list_remove_file_type(FileTypeList1,file_type_list_current(FileTypeList1));
  file_type_list_init(FileTypeList1);
  if(file_type_list_is_end(FileTypeList1)) 
    g_print("> FileTypeList1 is empty\n");
  else
    g_print("> FileTypeList1 is NOT empty = BUG\n");

  g_print("Free FileTypeList1\n");
  file_type_list_free(FileTypeList1);

  g_print("Content of FileTypeList2\n");
  file_type_list_init(FileTypeList2);
  while(file_type_list_is_end(FileTypeList2) == FALSE)
  {
    g_print("> %s\n",file_type_list_current(FileTypeList2)->type_str);
    file_type_list_next(FileTypeList2);
  }	
  
  g_print("Create a copy of FileTypeList2 in FileTypeList1\n");

  FileTypeList1 = file_type_list_copy(FileTypeList2);
  file_type_list_init(FileTypeList1);
  while(file_type_list_is_end(FileTypeList1) == FALSE)
  {
    g_print("> %s\n",file_type_list_current(FileTypeList1)->type_str);
    file_type_list_next(FileTypeList1);
  }	

  g_print("Free FileTypeList1\n");
  file_type_list_free(FileTypeList1);  
  
  g_print("Free FileTypeList2\n");
  file_type_list_free(FileTypeList2);  

  g_print("Create a FileTypeList2 with *.gif, *.bmp\n");
  FileTypeList2 = file_type_list_new();
  
  tmp_file_type = file_type_new(USER_DEFINED,"*.gif",NULL,
    "unknow.xpm",FALSE,NULL,NULL);
  file_type_list_append_file_type(FileTypeList2,tmp_file_type);
  tmp_file_type = file_type_new(USER_DEFINED,"*.bmp",NULL,
    "unknow.xpm",FALSE,NULL,NULL);
  file_type_list_append_file_type(FileTypeList2,tmp_file_type);

  tmp_file_type = file_type_list_find_user_defined_file_type(
  FileTypeList2,"teSt.gif",FALSE);
  if(tmp_file_type == NULL)
    g_print("> teSt.gif NOT identify (BUG)\n");
  else
    g_print("> teSt.gif identify with %s pattern\n",tmp_file_type->type_str);

  g_print("save FileTypeList2 in /root/programmation/GFile/test_file_type\n");
  file_type_list_save_to_file(FileTypeList2,
    "/root/programmation/GFile/test_file_type");

  g_print("Free FileTypeList2\n");
  file_type_list_free(FileTypeList2);

  g_print("load FileTypeList1 from /root/programmation/GFile/test_file_type2\n");
  FileTypeList1 = file_type_list_load_from_file("/root/programmation/GFile/test_file_type2");

  if(FileTypeList1 == NULL)
  { 
    g_print("ERROR in test_file_type2.\n"); 
    exit(1);
  }
  /* add all needed file types */
  file_type_list_add_needed_file_type(FileTypeList1);

  file_type_list_init(FileTypeList1);
  while(file_type_list_is_end(FileTypeList1) == FALSE)
  {
    g_print("> %25s, %8s, %8s, %8s, ",
      mode_str[file_type_list_current(FileTypeList1)->mode],
      file_type_list_current(FileTypeList1)->type_str,
      file_type_list_current(FileTypeList1)->program_str,
      file_type_list_current(FileTypeList1)->icon_name);
    if(file_type_list_current(FileTypeList1)->is_link)
      g_print("TRUE.\n");
    else
      g_print("FALSE.\n");
    file_type_list_next(FileTypeList1);
  }	

  g_print("Free FileTypeList1\n");
  file_type_list_free(FileTypeList1);
}

