/****************************************/
/* TestGFileList.c body                 */
/*					*/
/*  Description :Test the GFileList     */
/*               widget                 */
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

void file_mode_changed_cb (GtkWidget *Widget,FileData *SrcFile,
  FileData *DstFile, GFileList *pGFileList)
{
  g_print("<>Last : %s%s.\n",SrcFile->path,SrcFile->name);
  g_print("<>New  : %s%s.\n",DstFile->path,DstFile->name);  

/*  gfilelist_remove_file_with_check(pGFileList,SrcFile);
  gfilelist_add_file_with_check(pGFileList,DstFile);*/
  gfilelist_modify_file(pGFileList,SrcFile,DstFile);  
}    

void down_dir_cb(GFileList *pGFileList, gpointer data)
{
  g_print("go parent directory\n");
}

void up_dir_cb(GFileList *pGFileList,gchar *directory, gpointer data)
{
  g_print("go up directory \"%s\"\n",directory);
}

void scan_dir_cb(GFileList *pGFileList,FileDataList *pFileDataList, gpointer data)
{
  g_print("scan directory\n");
  file_data_list_init(pFileDataList);
  while(!file_data_list_is_end(pFileDataList))
  {
    g_print("<>%s\n",file_data_list_current(pFileDataList)->name);
    file_data_list_next(pFileDataList);
  }
}
       
int main(int argc, char **argv)
{ GtkWidget *vGFileList;
  GtkWidget *TestMain;
  Group *testGroup;
  Owner *testOwner;
  FileTypeList *testFileTypeList;

  gnome_init("TestGFileList", VERSION, argc,argv);

  testGroup = group_load();
  testOwner = owner_load(testGroup);  

  TestMain = gnome_app_new ("TestGFileList", "TestGFileList");
  gtk_widget_realize (TestMain);
  gtk_signal_connect (GTK_OBJECT (TestMain), "destroy",
                      GTK_SIGNAL_FUNC (gtk_main_quit),
                      NULL);
  file_type_set_parent(TestMain);

  testFileTypeList = file_type_list_new();
  /* we will have base internal type */
  file_type_list_add_needed_file_type(testFileTypeList);

  DefaultGFileListPref.owner_list = testOwner;
  DefaultGFileListPref.group_list = testGroup;
  DefaultGFileListPref.file_type_list = testFileTypeList;

  g_print("Create the new GFileList\n");  
  vGFileList = gfilelist_new(DefaultGFileListPref);
  g_print("New GFileList created\n");    

  gtk_signal_connect (GTK_OBJECT (vGFileList), "file_mode_changed",
                      GTK_SIGNAL_FUNC (file_mode_changed_cb),
                      vGFileList);

  gtk_signal_connect (GTK_OBJECT (vGFileList), "up_dir",
                      GTK_SIGNAL_FUNC (up_dir_cb),
                      NULL);
		      
  gtk_signal_connect (GTK_OBJECT (vGFileList), "down_dir",
                      GTK_SIGNAL_FUNC (down_dir_cb),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (vGFileList), "scan_dir",
                      GTK_SIGNAL_FUNC (scan_dir_cb),
                      NULL);
  
  gtk_widget_show(vGFileList);

  gnome_app_set_contents( GNOME_APP(TestMain), vGFileList);
		      
  gtk_widget_show(TestMain);
  
  g_print("Load the root content\n");  
  gfilelist_open_dir(GFILELIST(vGFileList),"/");
  g_print("Root content loaded\n");

  /* test add_file_by_full_name */
  system("touch /root/GFile.bidon");  
  gfilelist_add_file_by_full_name(GFILELIST(vGFileList),"/root/GFile.bidon");
  
  gtk_main();  

  system("rm /root/GFile.bidon");
  
  file_type_list_free(testFileTypeList);
  owner_free(testOwner);
  group_free(testGroup);  
}
